# #! /usr/bin/env python
# # -*- coding: utf-8 -*-
#
# """
# Module that contains data manager widgets
# """
#
# from __future__ import print_function, division, absolute_import
#
# import os
# import logging
# import pkgutil
# import inspect
# import traceback
#
# from tpDcc.core import data as core_data, scripts
#
# LOGGER = logging.getLogger('tpRigToolkit-core')
#
# _DIRECTORIES = list()
# _ASK_NAME_ON_CREATION = True
# _LOADED_DATA_CLASSES = list()
#
# STANDARD_DATA_CLASSES = [scripts.ScriptManifestData, scripts.ScriptPythonData]
#
#
# def get_all_data_classes(_reload=False):
#     """
#     Returns all data widgets loaded by the manager
#     :return: list<DataWidget>
#     """
#
#     return STANDARD_DATA_CLASSES + load_data_classes(_reload=_reload)
#
#
# def add_directory(directory, do_update=False):
#     """
#     Adds a new directory where data should be find
#     :param directory: str
#     :param do_update: bool
#     """
#
#     if directory not in _DIRECTORIES:
#         _DIRECTORIES.append(directory)
#         if do_update:
#             load_data_classes()
#
#
# def set_directories(directories):
#     """
#     Sets the directories where data should be find
#     :param directories: list(str)
#     """
#
#     new_dir = False
#     for d in directories:
#         if d not in directories:
#             new_dir = True
#             directories.append(d)
#
#     if new_dir:
#         load_data_classes()
#
#
# def load_data_classes(_reload=False):
#     """
#     Loads all data classes
#     :param _reload: bool
#     :return: list
#     """
#
#     for d in _DIRECTORIES:
#         loaded_classes = _load_data_classes(directory=d, _reload=_reload)
#         _LOADED_DATA_CLASSES.extend(loaded_classes)
#
#     return _LOADED_DATA_CLASSES
#
#
# def get_available_types():
#     """
#     Returns a list with all available data types
#     :return: list<str>
#     """
#
#     data_types = list()
#     for data in get_all_data_classes():
#         data_types.append(data.get_data_type())
#
#     return data_types
#
#
# def get_type_instance(data_type):
#     """
#     Returns a new instance of data type
#     :param data_type: str, type of data type instance we want to create
#     :return: variant
#     """
#
#     for data in get_all_data_classes():
#         if data.is_type_match(data_type):
#             return data()
#
#
# def _load_data_classes(directory, _reload=False):
#
#     imported = list()
#
#     if directory is None or not os.path.exists(directory):
#         LOGGER.warning('Data Path {} does not exists!'.format(directory))
#         return imported
#
#     for loader, mod_name, is_package in pkgutil.walk_packages([directory]):
#         try:
#             module = loader.find_module(mod_name).load_module(mod_name)
#             if _reload:
#                 reload(module)
#             for cname, obj in inspect.getmembers(module, inspect.isclass):
#                 if not issubclass(obj, core_data.FileData):
#                     continue
#                 # globals()[cname] = obj
#                 imported.append(obj)
#         except Exception as e:
#             LOGGER.warning('Aborting loading Data Class {} : {}'.format(mod_name, str(e)))
#             LOGGER.debug(traceback.format_exc())
#
#     return sorted(list(set(imported)))



#! /usr/bin/env python
# -*- coding: utf-8 -*-

"""
Module that contains implementation for names manager
"""

from __future__ import print_function, division, absolute_import

import logging

from tpDcc import dcc
from tpDcc.managers import configs
from tpDcc.libs.python import python
from tpDcc.libs.nameit.core import namelib

LOGGER = logging.getLogger('tpRigToolkit-core')


_NAME_LIB = None
_NAMING_FILE = None


def init_lib(naming_file=None, dev=True, cache=True):
    global _NAME_LIB
    global _NAMING_FILE
    if _NAME_LIB and naming_file and _NAMING_FILE == naming_file and cache:
        return _NAME_LIB

    environment = 'development' if dev else 'production'
    config = configs.get_config('tpRigToolkit-names', environment=environment)
    _NAMING_FILE = naming_file or config.get_path()
    _NAME_LIB = namelib.NameLib(naming_file=_NAMING_FILE)

    return _NAME_LIB


def get_auto_suffixes(dev=True):
    """
    Returns dictionary containing suffixes that can be used to handle nomenclature
    :return:
    """

    environment = 'development' if dev else 'production'
    naming_config = configs.get_config(config_name='tpRigToolkit-naming', environment=environment)
    if not naming_config:
        return dict()

    auto_suffixes_dict = naming_config.get('auto_suffixes', default=dict())
    if not auto_suffixes_dict:
        return dict()

    return auto_suffixes_dict


def parse_name(node_name, rule_name=None, naming_file=None, dev=False, cache=True):
    """
    Parse a current solved name and return its different fields (metadata information)
    :param node_name: str
    :param rule_name: str
    :param naming_file: str
    :param dev: bool
    :param cache: bool
    :return: dict(str)
    """

    name_lib = init_lib(naming_file=naming_file, dev=dev, cache=cache)
    rule_name = rule_name or 'default'
    rule = None
    if name_lib.has_rule(rule_name):
        rule = name_lib.get_rule(rule_name)
    if not rule:
        LOGGER.warning('Impossible to retrieve name because rule name "{}" is not defined!'.format(rule_name))
        return None

    current_rule = name_lib.active_rule()
    name_lib.set_active_rule(rule_name)
    parsed_name = name_lib.parse(node_name)
    if current_rule:
        if rule_name != current_rule.name:
            name_lib.set_active_rule(current_rule.name)
    else:
        name_lib.set_active_rule(None)

    return parsed_name


def solve_node_name_by_type(node_names=None, naming_file=None, dev=False, **kwargs):
    """
    Resolves node name taking into account its type
    In this case, the type of the node will be used to retrieve an an automatic rule
    The rule name will be retrieved using auto_suffix dict from tpDcc-naming configuration file
    :param node_names: str or list, name of the node we want to take name of
    :param naming_file: str
    :param dev: bool
    :return: str
    """

    if not dcc.is_maya():
        LOGGER.warning('Solve Node Name by type functionality is only supported in Maya!')
        return None

    import maya.cmds
    from tpDcc.dccs.maya.core import name

    name_lib = init_lib(naming_file=naming_file, dev=dev)

    auto_suffix = get_auto_suffixes()
    if not auto_suffix:
        LOGGER.warning(
            'Impossible to launch auto suffix functionality because no auto suffixes are defined!')
        return None

    solved_names = dict()
    return_names = list()

    if not node_names:
        node_names = dcc.selected_nodes()
    if not node_names:
        return
    node_names = python.force_list(node_names)

    for obj_name in node_names:
        obj_uuid = maya.cmds.ls(obj_name, uuid=True)
        if not obj_uuid:
            continue
        obj_uuid = obj_uuid[0]
        if obj_uuid in solved_names:
            LOGGER.warning(
                'Node with name: "{} and UUID "{}" already renamed to "{}"! Skipping ...'.format(
                    obj_name, obj_uuid, solved_names[obj_name]))
            continue

        # TODO: This code is a duplicated version of the one in
        #  tpDcc.dccs.maya.core.name.auto_suffix_object function. Move this code to a DCC specific function
        obj_type = maya.cmds.objectType(obj_name)
        if obj_type == 'transform':
            shape_nodes = maya.cmds.listRelatives(obj_name, shapes=True, fullPath=True)
            if not shape_nodes:
                obj_type = 'group'
            else:
                obj_type = maya.cmds.objectType(shape_nodes[0])
        elif obj_type == 'joint':
            shape_nodes = maya.cmds.listRelatives(obj_name, shapes=True, fullPath=True)
            if shape_nodes and maya.cmds.objectType(shape_nodes[0]) == 'nurbsCurve':
                obj_type = 'controller'
        if obj_type == 'nurbsCurve':
            connections = maya.cmds.listConnections('{}.message'.format(obj_name))
            if connections:
                for node in connections:
                    if maya.cmds.nodeType(node) == 'controller':
                        obj_type = 'controller'
                        break
        if obj_type not in auto_suffix:
            rule_name = 'node'
            node_type = obj_type
        else:
            rule_name = auto_suffix[obj_type]
            node_type = auto_suffix[obj_type]

        if not name_lib.has_rule(rule_name):
            if not name_lib.has_rule('node'):
                LOGGER.warning(
                    'Impossible to rename node "{}" by its type "{}" because rule "{}" it is not defined and '
                    'callback rule "node" either'.format(obj_name, obj_type, rule_name))
            else:
                rule_name = 'node'

        if rule_name == 'node':
            solved_name = solve_name(obj_name, node_type=node_type, rule_name=rule_name, **kwargs)
        else:
            solved_name = solve_name(rule_name, obj_name, rule_name=rule_name, **kwargs)
        solved_names[obj_uuid] = solved_name

    if not solved_names:
        return

    unique_name = kwargs.get('unique_name', False)

    for obj_id, solved_name in solved_names.items():
        obj_name = maya.cmds.ls(obj_id, long=True)[0]
        if not solved_names:
            return_names.append(obj_name)
        else:
            if unique_name:
                solved_name = dcc.find_unique_name(solved_name)
            new_name = name.rename(obj_name, solved_name, uuid=obj_id, rename_shape=True)
            return_names.append(new_name)

    return return_names


def solve_name(*args, **kwargs):
    """
    Resolves name with given rule and attributes
    :param args: list
    :param kwargs: dict
    """

    naming_file = kwargs.get('naming_file', None)
    dev = kwargs.get('dev', False)
    use_auto_suffix = kwargs.pop('use_auto_suffix', True)
    node_type = kwargs.get('node_type', None)
    rule_name = kwargs.pop('rule_name', None) or 'default'

    if use_auto_suffix and node_type:
        auto_suffixes = (get_auto_suffixes() or dict()).get(rule_name, dict())
        if node_type in auto_suffixes:
            kwargs['node_type'] = auto_suffixes[node_type]

    name_lib = init_lib(naming_file=naming_file, dev=dev)

    fallback_default = False
    if name_lib.has_rule(rule_name):
        rule = name_lib.get_rule(rule_name)
    else:
        LOGGER.warning('Rule with name "{}" is not defined. Default rule used'.format(rule_name))
        rule = name_lib.get_rule('default')
        fallback_default = True

    if not rule:
        LOGGER.warning('Impossible to retrieve name because rule name "{}" is not defined!'.format(rule_name))
        return None

    current_rule = name_lib.active_rule()
    name_lib.set_active_rule(rule)
    solved_name = name_lib.solve(*args, **kwargs)
    if current_rule:
        if rule_name != current_rule.name:
            name_lib.set_active_rule(current_rule.name)
    else:
        name_lib.set_active_rule(None)

    solved_name = solved_name or kwargs.get('default', None)

    unique_name = kwargs.get('unique_name', False)
    if solved_name and unique_name:
        solved_name = dcc.find_unique_name(solved_name)

    if fallback_default:
        LOGGER.warning('Solved name with no rule (used default one): {}'.format(solved_name))

    return solved_name



#! /usr/bin/env python
# -*- coding: utf-8 -*-

"""
Module that contains Control CV position data implementation
"""

from __future__ import print_function, division, absolute_import

import os
import re
import logging

from tpDcc import dcc
from tpDcc.core import dcc as core_dcc
from tpDcc.libs.python import python, decorators, fileio, folder, jsonio, path as path_utils
from tpDcc.libs.datalibrary.core import consts, datapart
from tpDcc.libs.curves.core import curveslib

from tpRigToolkit.core import utils
from tpRigToolkit.data import curves
from tpRigToolkit.libs.controlrig.core import controllib

logger = logging.getLogger(consts.LIB_ID)


class _MetaControlCVsLib(type):

    def __call__(self, *args, **kwargs):
        if dcc.client().is_maya():
            from tpRigToolkit.dccs.maya.data import controlcv
            return type.__call__(controlcv.ControlCVsLibMaya, *args, **kwargs)
        else:
            return type.__call__(BaseControlCVsLib, *args, **kwargs)


class BaseControlCVsLib(object):
    def __init__(self):
        self._libraries = dict()
        self._library_curves = dict()
        self._active_library = None
        self._curves_data_path = None
        self._extension = ControlCVsData.EXTENSION

        self._load_libraries()

    # ==============================================================================================
    # ABSTRACT
    # ==============================================================================================

    @decorators.abstractmethod
    def _get_curve_transform(self, curve):
        pass

    # ==============================================================================================
    # BASE
    # ==============================================================================================

    def set_directory(self, directory_path):
        self._curves_data_path = directory_path
        self._libraries = dict()
        self._load_libraries()
        self._library_curves = dict()
        self._initialize_library_curve()

    def get_active_library(self):
        return self._active_library

    def set_active_library(self, library_name, skip_extension=False):
        if not skip_extension:
            file_name = '{}{}'.format(library_name, self._extension)
        else:
            file_name = library_name

        library_path = fileio.create_file(file_name, self._curves_data_path)
        self._active_library = library_name
        self._library_curves[library_name] = dict()
        if skip_extension:
            self.load_data_file(library_path)
        else:
            self.load_data_file()

    def get_library_names(self):
        return list(self._libraries.keys())

    def get_curve_names(self):
        if not self._active_library:
            logger.warning('Must set active library before running this function')
            return

        return self._library_curves[self._active_library].keys()

    def load_data_file(self, file_path=None):
        if not self._active_library:
            logger.warning('Must set active library before running this function.')
            return
        if not file_path:
            file_path = path_utils.join_path(
                self._curves_data_path, '{}{}'.format(self._active_library, self._extension))

        data = jsonio.read_file(file_path) or dict()
        for curve_name, curve_data in data.items():
            self._library_curves[self._active_library][curve_name] = curve_data

    def write_data_to_file(self):
        if not self._active_library:
            logger.warning('Must set active library before running this function')
            return

        file_path = path_utils.join_path(self._curves_data_path, self._active_library)
        if not file_path.endswith(self._extension):
            file_path = '{}{}'.format(file_path, self._extension)

        library_data = self._library_curves[self._active_library]
        jsonio.write_to_file(library_data, file_path)

        return file_path

    def add_curve(self, curve, library_name=None):
        if not curve:
            return
        if library_name:
            self.set_active_library(library_name)
        else:
            library_name = self._active_library
            if not self._active_library:
                logger.warning('Must set active library before running this function')
                return

        transform = self._get_curve_transform(curve)
        curve_data = curveslib.serialize_curve(transform, normalize=False)
        self._update_curve_data(transform, curve_data)
        self._library_curves[library_name][transform] = curve_data

    def remove_curve(self, curve, library_name=None):
        if not curve:
            return False

        if not library_name:
            library_name = self._active_library
            if not self._active_library:
                logger.warning('Must set active library before running this function')
                return False

        transform = self._get_curve_transform(curve)
        if library_name in self._library_curves:
            if transform in self._library_curves[library_name]:
                self._library_curves[library_name].pop(transform)
                return True

        return False

    def set_curve(self, curve, curve_in_library=None, check_curve=False):
        if not self._active_library:
            logger.warning('Must set active library before running this function')
            return

        curve_in_library = curve_in_library or curve

        curve_data = self._get_curve_data(curve_in_library, self._active_library)
        if not curve_data:
            return

        if check_curve:
            is_curve = self._is_curve_of_type(curve, curve_in_library)
            if not is_curve:
                logger.warning('curve data does not store expected curve type')
                return False

        transform = self._get_curve_transform(curve)
        controllib.replace_control_curves(transform, control_data=curve_data, auto_scale=False)

    # ==============================================================================================
    # INTERNAL
    # ==============================================================================================

    def _get_curves_data_path(self):
        current_path = curves.__path__[0]
        custom_curve_path = utils.get_custom('custom_curve_path')
        if custom_curve_path and os.path.isdir(custom_curve_path):
            curve_data = os.path.join(custom_curve_path)
            folder.create_folder(curve_data)
            logger.info('Using custom curve directory: {}'.format(custom_curve_path))
            current_path = custom_curve_path

        self._curves_data_path = current_path

        return self._curves_data_path

    def _load_libraries(self):
        curves_data_path = self._curves_data_path or self._get_curves_data_path()
        files = os.listdir(curves_data_path)
        for filename in files:
            if filename.endswith(self._extension):
                split_file = filename.split('.')
                self._library_curves[split_file[0]] = filename

    def _initialize_library_curve(self):
        names = self.get_library_names()
        for name in names:
            self._library_curves[name] = dict()

    def _update_curve_data(self, transform, curve_data):
        if dcc.attribute_exists(transform, 'curveType') or dcc.attribute_exists(transform, 'type'):
            value = dcc.get_attribute_value(transform, 'curveType')
            if value:
                curve_data['type'] = value
        if dcc.attribute_exists(transform, 'type'):
            value = dcc.get_attribute_value(transform, 'type')
            if value:
                curve_data['type'] = value

    def _get_curve_data(self, curve_name, curve_library):
        curve_dict = self._library_curves[curve_library]
        if curve_name not in curve_dict:
            # logger.warning('{} is not in the curve library {}'.format(curve_name, curve_library))
            return dict()

        return curve_dict[curve_name]

    def _get_curve_type(self, curve):
        curve_type_value = None
        if dcc.attribute_exists(curve, 'curveType'):
            curve_type_value = dcc.get_attribute_value(curve, 'curveType')
        elif dcc.attribute_exists(curve, 'type'):
            curve_type_value = dcc.get_attribute_value(curve, 'type')

        return curve_type_value

    def _is_curve_of_type(self, curve, type_curve):
        curve_data = self._get_curve_data(type_curve, self._active_library)
        if not curve_data:
            return False
        original_curve_type = curve_data.get('curveType', curve_data.get('type', ''))
        if not original_curve_type:
            return True
        curve_type_value = self._get_curve_type(curve)
        if curve_type_value and curve_type_value != original_curve_type:
            return False

        return True


class ControlCVsData(datapart.DataPart):
    DATA_TYPE = 'dcc.controlcvs'
    MENU_ICON = 'circle'
    MENU_NAME = 'Control CVs'
    PRIORITY = 13
    EXTENSION = '.cvs'

    _has_trait = re.compile(r'\.cvs$', re.I)

    @classmethod
    def can_represent(cls, identifier, only_extension=False):
        if ControlCVsData._has_trait.search(identifier):
            if only_extension:
                return True
            if os.path.isfile(identifier):
                return True

        return False

    @classmethod
    def supported_dccs(cls):
        return [core_dcc.Dccs.Maya]

    def label(self):
        return os.path.basename(self.identifier())

    def icon(self):
        return 'circle'

    def extension(self):
        return '.cvs'

    def type(self):
        return 'dcc.controlcvs'

    def menu_name(self):
        return 'Control CVs'

    def save_schema(self):
        """
        Returns the schema used for saving the item
        :return: dict
        """

        return [
            {
                'name': 'objects',
                'type': 'objects',
                'layout': 'vertical',
                'errorVisible': True
            }
        ]

    def save_validator(self, **kwargs):
        """
        Validates the given save fields
        Called when an input field has changed
        :param kwargs: dict
        :return: list(dict)
        """

        fields = list()

        selection = dcc.client().selected_nodes() or list()
        msg = ''
        if not selection:
            msg = 'No objects selected. Please select at least one object.'

        fields.append({'name': 'objects', 'value': selection, 'error': msg})

        return fields

    def functionality(self):
        return dict(
            save=self.save,
            import_data=self.import_data,
            export_data=self.export_data,
            get_curves=self.get_curves,
            remove_curve=self.remove_curve
        )

    def save(self, *args, **kwargs):

        filepath = self.format_identifier()
        if not filepath.endswith(ControlCVsData.EXTENSION):
            filepath = '{}{}'.format(filepath, ControlCVsData.EXTENSION)

        if not filepath:
            logger.warning('Impossible to save Control CVs file because save file path not defined!')
            return

        logger.debug('Saving {} | {}'.format(filepath, kwargs))

        objects = kwargs.get('objects', None)
        if not objects:
            objects = dcc.client().selected_nodes(full_path=True)
        if not objects:
            logger.warning(
                'Nothing selected to export Control CVs of. Please, select a curve to export')
            return False

        # We make sure that we store the short name of the controls
        objects = [dcc.client().node_short_name(obj) for obj in objects]

        controls = objects or controllib.get_controls()
        valid_controls = list()
        for control in controls:
            if not controllib.is_control(control):
                continue
            valid_controls.append(control)
        if not valid_controls:
            logger.warning('No valid controls found to export.')
            return False

        library = self._initialize_library(filepath)
        if not controls:
            logger.warning('No controls found to export.')
            return False

        for control in controls:
            library.add_curve(control)

        library.write_data_to_file()

        logger.info('Saved {} data'.format(self.name()))

        return True

    def import_data(self, **kwargs):

        filepath = self.format_identifier()
        if not filepath.endswith(ControlCVsData.EXTENSION):
            filepath = '{}{}'.format(filepath, ControlCVsData.EXTENSION)

        if not filepath:
            logger.warning('Impossible to load Control CVs from file: "{}"!'.format(filepath))
            return False

        library = self._initialize_library(filepath)

        objects = kwargs.get('objects', None)
        if not objects:
            objects = dcc.client().selected_nodes(full_path=True) or list()

        # We make sure that we store the short name of the controls
        objects = [dcc.node_short_name(obj) for obj in objects]

        updated_controls = list()
        controls = objects or controllib.get_controls()
        for control in controls:
            shapes = dcc.client().get_curve_shapes(control)
            if not shapes:
                continue
            library.set_curve(control, check_curve=True)
            updated_controls.append(control)

        # We force the update of all the curves that are stored in the library and are in the scene
        for curve_data in list(library._library_curves.values()):
            for curve_name in list(curve_data.keys()):
                if curve_name and dcc.node_exists(curve_name) and curve_name not in updated_controls:
                    library.set_curve(curve_name, check_curve=True)
                    updated_controls.append(curve_name)

        logger.info('Imported {} data'.format(self.name()))

        return True

    def export_data(self, **kwargs):
        filepath = self.format_identifier()
        if not filepath.endswith(ControlCVsData.EXTENSION):
            filepath = '{}{}'.format(filepath, ControlCVsData.EXTENSION)

        if not filepath or not os.path.isfile(filepath):
            logger.warning('Impossible to export Control CVs data to: "{}"'.format(filepath))
            return

        objects = kwargs.get('objects', list())
        selected_objects = dcc.client().selected_nodes(full_path=True)
        objects = list(set(objects + selected_objects))
        kwargs['objects'] = objects

        return self.save(**kwargs)

    def get_curves(self, file_name=None):
        library = self._initialize_library(file_name)
        library.set_active_library(self.name)
        curves = library.get_curve_names()

        return curves

    def remove_curve(self, curve_name, file_name=None):
        curves_list = python.force_list(curve_name)
        library = self._initialize_library(file_name)
        library.set_active_library(self.name)
        for curve_found in curves_list:
            library.remove_curve(curve_found)
        library.write_data_to_file()

        return True

    def _initialize_library(self, file_name=None):
        if file_name:
            directory = path_utils.get_dirname(file_name)
            name = path_utils.get_basename(file_name)
        else:
            path = self.format_identifier()
            directory = path_utils.get_dirname(path)
            name = self.name()

        library = ControlCVsLib()
        library.set_directory(directory)

        if file_name:
            library.set_active_library(name, skip_extension=True)
        else:
            library.set_active_library(name)

        return library


@decorators.add_metaclass(_MetaControlCVsLib)
class ControlCVsLib(object):
    pass


# class ControlCVOptionsWidget(loadwidget.OptionsFileWidget, object):
#     def __init__(self, parent=None):
#         super(ControlCVOptionsWidget, self).__init__(parent=parent)
#
#     def ui(self):
#         super(ControlCVOptionsWidget, self).ui()
#
#         self._search_line = search.SearchFindWidget(parent=self)
#         self._search_line.set_placeholder_text('Filter Names')
#         self._curves_list = QListWidget()
#         self._curves_list.setSizePolicy(QSizePolicy.Expanding, QSizePolicy.Expanding)
#         self._curves_list.setSelectionMode(self._curves_list.ExtendedSelection)
#         self._curves_list.setSortingEnabled(True)
#         self._delete_curve_button = buttons.BaseButton('Delete Curve CVs data', parent=self)
#
#         self.main_layout.addWidget(self._search_line)
#         self.main_layout.addWidget(self._curves_list)
#         self.main_layout.addWidget(dividers.Divider())
#         self.main_layout.addWidget(self._delete_curve_button)
#
#     def setup_signals(self):
#         self._search_line.textChanged.connect(self._on_filter_names)
#         self._delete_curve_button.clicked.connect(self._on_remove_curves)
#
#     def refresh(self):
#         self._curves_list.clear()
#
#         if not self._data_object:
#             return
#
#         curves = self._data_object.get_curves()
#         if not curves:
#             return
#
#         for curve_name in curves:
#             item = QListWidgetItem(curve_name)
#             self._curves_list.addItem(item)
#
#     def _on_filter_names(self):
#         self._unhide_names()
#         for i in range(self._curves_list.count()):
#             item = self._curves_list.item(i)
#             text = str(item.text())
#             filter_text = self._search_line.text()
#             if text.find(filter_text) == -1:
#                 item.setHidden(True)
#
#     def _on_remove_curves(self):
#         items = self._curves_list.selectedItems()
#         if not items:
#             return
#
#         for item in items:
#             curve_name = str(item.text())
#             removed = self._data_object.remove_curve(curve_name)
#             if removed:
#                 index = self._curves_list.indexFromItem(item)
#                 remove_item = self._curves_list.takeItem(index.row())
#                 del remove_item
#
#     def _unhide_names(self):
#         for i in range(self._curves_list.count()):
#             item = self._curves_list.item(i)
#             item.setHidden(False)



#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""
Base wrapper classes to create DCC dialogs
"""

from __future__ import print_function, division, absolute_import

__author__ = "Tomas Poveda"
__license__ = "MIT"
__maintainer__ = "Tomas Poveda"
__email__ = "tpovedatd@gmail.com"

from tpDcc.dcc import dialog
from tpDcc.managers import resources


class MainDialog(dialog.Dialog, object):

    def __init__(
            self,
            tool=None,
            name='MainDialog',
            title='MainDialog',
            show_dragger=True, size=(200, 125),
            fixed_size=False,
            parent=None):

        self._tool = tool

        super(MainDialog, self).__init__(
            name=name,
            title=title,
            parent=parent,
            show_dragger=show_dragger,
            size=size,
            fixed_size=fixed_size
        )

    def ui(self):
        super(MainDialog, self).ui()

        dialog_icon = self._get_icon()
        self.setWindowIcon(dialog_icon)

        if self._tool:
            self.main_layout.addWidget(self._tool)

        # if self._project.is_dev():
        #     int_colors = self._project.dev_color0.split(',')
        #     dev_style = "background-color: rgb({}, {}, {})".format(
        #         int_colors[0], int_colors[1], int_colors[2], 255)
        #     self._dragger.setStyleSheet(dev_style)

    def setWindowTitle(self, title):
        # if self._project.is_dev():
        #     title = '{} - [{}]'.format(title, self._project.get_environment())

        super(MainDialog, self).setWindowTitle(title)

    def _get_icon(self):
        """
        Internal function that returns the icon used for the window
        :return: QIcon
        """

        if self._project:
            window_icon = self._project.icon
            if not window_icon.isNull():
                return window_icon
            else:
                self._project.logger.warning(
                    '{} Project Icon not found: {}!'.format(
                        self._project.name.title(), self._project.icon_name + '.png'))

        return resources.icon('tpdcc')



#! /usr/bin/env python
# -*- coding: utf-8 -*-

"""
Module that contains options to select rig controls
"""

from __future__ import print_function, division, absolute_import

import os


from Qt.QtCore import Qt, Signal
from Qt.QtWidgets import QDialog

from tpDcc.managers import resources
from tpDcc.libs.qt.core import base
from tpDcc.libs.qt.widgets import layouts, label, lineedit, buttons

from tpRigToolkit.widgets.options import rigoption


class RigControlOption(rigoption.RigOption, object):
    def __init__(self, name, parent, main_widget, rig_object):
        super(RigControlOption, self).__init__(name=name, parent=parent, main_widget=main_widget, rig_object=rig_object)

    def get_option_type(self):
        return 'rigcontrol'

    def get_option_widget(self):
        return GetControlRigWidget(name=self._name, rig_object=self._rig)

    def get_name(self):
        name = self._option_widget.get_name()
        return name

    def set_name(self, name):
        self._option_widget.set_name(name)

    def get_value(self):
        return self._option_widget.control_data

    def set_value(self, value):
        self._option_widget.set_value(value)

    def _setup_option_widget_value_change(self):
        self._option_widget.valueChanged.connect(self._on_value_changed)


class GetControlRigWidget(base.BaseWidget, object):
    valueChanged = Signal(object)

    def __init__(self, name, rig_object, parent=None):
        self._name = name
        self._rig = rig_object
        super(GetControlRigWidget, self).__init__(parent=parent)

        self._control_data = dict()

    @property
    def control_data(self):
        return self._control_data

    def get_main_layout(self):
        main_layout = layouts.HorizontalLayout()
        main_layout.setSpacing(2)
        main_layout.setContentsMargins(2, 2, 2, 2)

        return main_layout

    def ui(self):
        super(GetControlRigWidget, self).ui()

        self._label = label.BaseLabel(self._name)
        self._label.setAttribute(Qt.WA_TransparentForMouseEvents)
        self._control = ControlLineEdit(self._rig)

        self.main_layout.addWidget(self._label)
        self.main_layout.addWidget(self._control)

    def setup_signals(self):
        self._control.controlSelected.connect(self._on_selected_control)

    def get_value(self):
        return self._control_data

    def set_value(self, value_dict):
        self._control_data = value_dict or dict()
        self._control.set_data(self._control_data)

    def get_name(self):
        return self._label.text()

    def set_name(self, value):
        self._label.setText(value)

    def _on_selected_control(self, control_data):
        self.set_value(control_data)
        self.valueChanged.emit(control_data)


class ControlLineEdit(base.BaseWidget, object):
    controlSelected = Signal(object)

    def __init__(self, rig_object, parent=None):
        self._rig_object = rig_object
        self._control_data = dict()
        super(ControlLineEdit, self).__init__(parent=parent)

    @property
    def control_data(self):
        return self._control_data

    def get_main_layout(self):
        main_layout = layouts.HorizontalLayout()
        main_layout.setSpacing(2)
        main_layout.setContentsMargins(2, 2, 2, 2)

        return main_layout

    def ui(self):
        super(ControlLineEdit, self).ui()

        self._line = lineedit.BaseLineEdit()
        self._btn = buttons.BaseButton(text='...')

        self.main_layout.addWidget(self._line)
        self.main_layout.addWidget(self._btn)

    def setup_signals(self):
        self._btn.clicked.connect(self._on_open_rig_control_selector)

    def set_data(self, data):
        data = data if data is not None else dict()
        name = data.get('control_type', '')
        self._line.setText(str(name))
        self._line.setToolTip(str(data))
        self._control_data = data

    def _on_open_rig_control_selector(self):

        from tpRigToolkit.tools.controlrig.core import client, tool, toolset

        dlg = QDialog(parent=self)
        dlg.setWindowTitle('Select Control')
        dlg.setWindowIcon(resources.icon('circle'))
        lyt = layouts.VerticalLayout(spacing=0, margins=(0, 0, 0, 0))
        lyt.setSpacing(0)
        lyt.setContentsMargins(0, 0, 0, 0)
        dlg.setLayout(lyt)

        if hasattr(self._rig_object, 'full_path'):
            current_project = self._rig_object
        else:
            current_project = self._rig_object.get_project()
        project_path = current_project.full_path

        # TODO: The project itself should return this path
        controls_path = os.path.join(project_path, 'controls.json')

        self._control_rig_client = client.ControlRigClient.create(tool.ControlRigTool.ID)
        control_selector = toolset.ControlRigToolset(
            as_selector=True, controls_path=controls_path, control_data=self._control_data,
            selector_parent=dlg)
        control_selector.initialize(self._control_rig_client)

        lyt.addWidget(control_selector)
        dlg.resize(600, 700)
        dlg.exec_()
        control_data = control_selector.control_data or dict()
        if not control_data and self._control_data:
            return
        if control_data:
            control_data.pop('control_name', None)
            # control_data.pop('control_data', None)

        self.controlSelected.emit(control_data)



#! /usr/bin/env python
# -*- coding: utf-8 -*-

"""
Module that contains rig base control implementation
"""

import logging

from tpDcc import dcc
from tpDcc.libs.python import decorators

from tpRigToolkit.managers import names
from tpRigToolkit.libs.controlrig.core import controllib

logger = logging.getLogger('tpRigToolkit-core')


class _MetaRigControl(type):

    def __call__(self, *args, **kwargs):
        if dcc.client().is_maya():
            from tpRigToolkit.dccs.maya.core import control
            return type.__call__(control.MayaRigControl, *args, **kwargs)
        else:
            return type.__call__(BaseRigControl, *args, **kwargs)


class BaseRigControl(object):

    axis_map = {
        'X': 'XYZ',
        'Y': 'YXZ',
        'Z': 'ZYX'
    }

    def __init__(self, name, **kwargs):
        super(BaseRigControl, self).__init__()

        self._name = name or 'newCtrl'
        self._curve_type = kwargs.pop('control_type', '')
        self._controls_path = kwargs.pop('controls_path', '')
        self._naming_file = kwargs.pop('naming_file', '')
        self._name_rule = kwargs.pop('rule_name', '')
        self._name_data = kwargs.pop('name_data', dict())
        self._side = kwargs.pop('side', None)

        if self._naming_file:
            self._name = self.solve_name(self._name, unique_name=False)

        if not dcc.node_exists(self._name):
            self._create(**kwargs)

    def __repr__(self):
        return '{} | {}'.format(self.__class__.__name__, self._name)

    # =================================================================================================================
    # PROPERTIES
    # =================================================================================================================

    @property
    def name(self):
        return self._name

    @property
    def side(self):
        return self._side

    # =================================================================================================================
    # ABSTRACT
    # =================================================================================================================

    @decorators.abstractmethod
    def get_rgb_color(self, linear=True):
        """
        Returns the RGB color of the given control, looking in the first shape node
        :param linear: bool, Whether or not the RGB should be in linear space (matches viewport color)
        :return: tuple(float, float, float), new control color in float linear values (between 0.0 and 1.0)
        """

        return 1.0, 1.0, 1.0

    @decorators.abstractmethod
    def set_color(self, value):
        """
        Sets the color of the control shapes
        :param value: int or list, color defined by its RGB color or by its index
        """

        pass

    @decorators.abstractmethod
    def translate_cvs(self, x, y, z):
        """
        Translates control curve CVs in object space
        :param x: float
        :param y: float
        :param z: float
        """

        pass

    @decorators.abstractmethod
    def rotate_cvs(self, x, y, z):
        """
        Rotates control curve CVs in object space
        :param x: float
        :param y: float
        :param z: float
        """

        pass

    @decorators.abstractmethod
    def scale_cvs(self, x, y, z, use_pivot=True):
        """
        Scales control curve CVs in object space
        :param x: float
        :param y: float
        :param z: float
        :param use_pivot: bool
        """

        pass

    # =================================================================================================================
    # BASE
    # =================================================================================================================

    def get(self):
        """
        Returns name of the control
        :return: str
        """

        return self._name

    def get_top(self):
        """
        Returns top control (taking into account root and auto buffer groups)
        :return: str
        """

        # Used by DCCs that cannot store transform offset inside the "transform" node such as Maya

        return self.get()

    def has_root_buffer(self):
        """
        Returns whether or not control has a a root buffer
        :return: bool
        """

        # Used by DCCs that cannot store transform offset inside the "transform" node such as Maya

        pass

    def has_auto_buffer(self):
        """
        Returns whether or not control has a a root buffer
        :return: bool
        """

        # Used by DCCs that cannot store transform offset inside the "transform" node such as Maya

        pass

    def create_root(self, name=None, *args, **kwargs):
        """
        Creates a root buffer group above the control
        :return: str
        """

        # Used by DCCs that cannot store transform offset inside the "transform" node such as Maya

        pass

    def create_auto(self, name=None, *args, **kwargs):
        """
        Creates an auto buffer group above the control
        :return: str
        """

        # Used by DCCs that cannot store transform offset inside the "transform" node such as Maya

        pass

    def set_parent(self, parent, **kwargs):
        """
        Sets the parent of the control
        :param parent:
        """

        dcc.set_parent(self.get_top(), parent)

    def set_naming_file(self, naming_file, name_rule=None):
        """
        Sets the naming file and rule used to manage the nomenclature of the control
        :param naming_file: str
        :param name_rule: str
        :return:
        """

        self._naming_file = naming_file
        self._name_rule = name_rule

    def set_curve_type(self, type_name=None, keep_color=True, **kwargs):
        """
        Updates the curves of the control with the given control type
        :param type_name: str
        :param keep_color: bool
        """

        color = kwargs.get('color', None)
        control_size = kwargs.get('control_size', 1.0)
        auto_scale = kwargs.get('auto_scale', True)
        auto_scale = auto_scale if control_size is None else False
        axis = kwargs.pop('axis', None)
        if axis:
            kwargs['axis_order'] = self.axis_map.get(axis, None)
        if color:
            keep_color = False
        control_type = kwargs.pop('control_type', 'circle')
        kwargs['control_type'] = type_name or control_type
        kwargs['keep_color'] = keep_color
        kwargs['auto_scale'] = auto_scale
        kwargs['parent'] = self._name

        new_control = controllib.replace_control_curves(self._name, **kwargs)

        dcc.clear_selection()

        return new_control

    def solve_name(self, *args, **kwargs):
        """
        Solves the given name using stored control naming file and rule
        :param args:
        :param kwargs:
        :return:
        """

        kwargs['naming_file'] = self._naming_file
        kwargs['rule_name'] = self._name_rule
        node_id = kwargs.pop('id', None)
        if not node_id:
            node_id = self._name_data.pop('id', None)
        node_type = kwargs.pop('node_type', 'control')
        kwargs.update(self._name_data)

        solved_name = names.solve_name(node_type=node_type, id=node_id, *args, **kwargs)

        return solved_name

    def parse_name(self, name):
        """
        Parse a current solved name and return its different fields (metadata information)
        :param name: str
        :return: dict(str)
        """

        return names.parse_name(node_name=name, rule_name=self._name_rule, naming_file=self._naming_file)

    def match_translation(self, target):
        """
        Matches control translation to given target
        :param target:
        """

        dcc.match_translation(target, self.get_top())

    def match_rotation(self, target):
        """
        Matches control rotation to given target
        :param target:
        """

        dcc.match_rotation(target, self.get_top())

    def match_scale(self, target):
        """
        Matches control scale to given target
        :param target:
        """

        dcc.match_scale(target, self.get_top())

    def match_translation_rotation(self, target):
        """
        Matches control translation and scale to given target
        :param target:
        """

        dcc.match_translation_rotation(target, self.get_top())

    def match_transform(self, target):
        """
        Matches control translation, rotation and scale to given target
        :param target:
        """

        dcc.match_transform(target, self.get_top())

    def hide_translate_attributes(self):
        """
        Locks and hide translate attributes on the control
        """

        dcc.hide_translate_attributes(self.get())

    def hide_rotate_attributes(self):
        """
        Locks and hide rotate attributes on the control
        """

        dcc.hide_rotate_attributes(self.get())

    def hide_scale_attributes(self):
        """
        Locks and hide scale attributes on the control
        """

        dcc.hide_scale_attributes(self.get())

    def hide_visibility_attribute(self):
        """
        Locks and hide the visibility attribute on the control
        """

        dcc.hide_visibility_attribute(self.get())

    def hide_scale_and_visibility_attributes(self):
        """
        Locks and hide the visibility and scale attributes on the control
        """

        self.hide_scale_attributes()
        self.hide_visibility_attribute()

    def hide_keyable_attributes(self):
        """
        Locks and hide all keyable attributes on the control
        """

        dcc.hide_keyable_attributes(self.get())

    # =================================================================================================================
    # INTERNAL
    # =================================================================================================================

    def _get_name(self, *args, **kwargs):
        """
        Internal function that returns a proper name for elements of the rig control
        :param name: str
        :param node_type: str
        :return: str
        """

        name_data = self._name_data.copy()
        name_data.update(**kwargs)
        return names.solve_name(naming_file=self._naming_file, rule_name=self._name_rule, *args, **name_data)

    def _create(self, **kwargs):
        """
        Internal function that forces the creation of the control curve
        """

        return self.set_curve_type(type_name=self._curve_type, keep_color=False, **kwargs)


@decorators.add_metaclass(_MetaRigControl)
class RigControl(object):
    pass



#! /usr/bin/env python
# -*- coding: utf-8 -*-

"""
Module that contains utils functions for tpRigToolkit
"""

from __future__ import print_function, division, absolute_import

import logging

from tpDcc.libs.python import folder, fileio, version, path as path_utils

LOGGER = logging.getLogger('tpRigToolkit-core')


def copy(source, target, description=''):
    """
    Copies given file or files and creates a new version of the file with the given description
    :param source: str, source file or folder we want to copy
    :param target: str, destination file or folder we want to copy into
    :param description: str, description of the new version
    """

    is_source_a_file = path_utils.is_file(source)

    if is_source_a_file:
        copied_path = fileio.copy_file(source, target)
    else:
        if not path_utils.exists(source):
            LOGGER.info('Nothing to copy: {}\t\tData was probably created but not saved yet.'.format(
                path_utils.get_dirname(is_source_a_file)))
            return
        if path_utils.exists(target):
            folder.delete_folder(target)
        copied_path = folder.copy_folder(source, target)

    if not copied_path:
        LOGGER.warning('Error copying {}\t to\t{}'.format(source, target))
        return

    if copied_path > -1:
        LOGGER.info('Finished copying {} from {} to {}'.format(description, source, target))
        version_file = version.VersionFile(copied_path)
        version_file.save('Copied from {}'.format(source))


def get_custom(name, default=''):
    """
    Returns custom attributte defined in tpRigToolkit __custom__ module
    This module can be override by users if necessary
    :param name: str
    :param default: str
    :return: object
    """

    try:
        from tpRigToolkit import __custom__
    except Exception:
        return default

    value = None
    exec('value = __custom__.{}'.format(name))
    if not value:
        return default

    return value



#! /usr/bin/env python
# -*- coding: utf-8 -*-

"""
Module that contains renamer tool implementation
"""

from __future__ import print_function, division, absolute_import

from Qt.QtCore import Qt
from Qt.QtWidgets import QSizePolicy, QWidget

from tpDcc.core import plugin
from tpDcc.managers import configs, resources
from tpDcc.libs.qt.widgets import layouts
from tpDcc.tools.renamer.core import renamer


class RenamerPlugin(plugin.DockPlugin, object):

    NAME = 'Renamer'
    TOOLTIP = 'Allows to rename DCC nodes'
    DEFAULT_DOCK_AREA = Qt.LeftDockWidgetArea
    IS_SINGLETON = True

    def __init__(self):
        super(RenamerPlugin, self).__init__()

        self._renamer_widget = None
        self._content = QWidget()
        self._content_layout = layouts.VerticalLayout(spacing=0, margins=(0, 0, 0, 0))
        self._content.setLayout(self._content_layout)
        self.setWidget(self._content)

    @staticmethod
    def icon():
        return resources.icon('rename')

    def show_plugin(self):
        super(RenamerPlugin, self).show_plugin()

        # TODO: This should be defined
        dev = False

        names_config = configs.get_config(
            config_name='tpRigToolkit-names', environment='development' if dev else 'production')
        naming_config = configs.get_config(
            config_name='tpRigToolkit-naming', environment='development' if dev else 'production')

        self._renamer_widget = renamer.RenamerToolsetWidget(
            names_config=names_config, naming_config=naming_config, parent=self)
        self._renamer_widget.initialize()
        self._renamer_widget.setSizePolicy(QSizePolicy.Expanding, QSizePolicy.Expanding)
        self._content_layout.addWidget(self._renamer_widget)



# #! /usr/bin/env python
# # -*- coding: utf-8 -*-
#
# """
# Module that contains data manager implementation
# """
#
# from __future__ import print_function, division, absolute_import
#
# import os
# import logging
# import pkgutil
# import inspect
# import traceback
#
# from tpDcc.libs.python import path, decorators
# # from tpDcc.libs.qt.widgets.library import manager
#
# from tpRigToolkit.core import data, utils
#
# LOGGER = logging.getLogger('tpRigToolkit-core')
#
#
# @decorators.add_metaclass(decorators.Singleton)
# class DataManager(manager.LibraryManager, object):
#     def __init__(self, settings=None, directories=None, update_on_init=True):
#         super(DataManager, self).__init__(settings=settings)
#
#         data_dirs = utils.get_data_files_directory()
#         data_dirs.extend(directories or list())
#
#         self._directories = list(set(data_dirs))
#         self._loaded_data_items = list()
#         self._loaded_data_classes = list()
#
#         self.standard_data_classes = [
#             scripts.ScriptManifestData,
#             scripts.ScriptPythonData
#         ]
#
#         if update_on_init:
#             self.update_data_classes()
#             self.update_data_items()
#
#     # def get_all_data_classes(self, do_reload=False):
#     #     """
#     #     Returns all data classes loaded by the manager
#     #     :return: list<DataWidget>
#     #     """
#     #
#     #     return self.standard_data_classes + self.update_data_classes(do_reload=do_reload)
#
#     def get_all_data_items(self, do_reload=False):
#         """
#         Returns all data item classes loaded by the manager
#         :param do_reload:
#         :return:
#         """
#
#         return self.update_data_items(do_reload=do_reload)
#
#     def get_available_types(self):
#         """
#         Returns a list with all available data types
#         :return: list<str>
#         """
#
#         data_types = list()
#         for data_class in self.get_all_data_classes():
#             data_types.append(data_class.get_data_type())
#
#         return data_types
#
#     # def add_directory(self, directory, do_update=False):
#     #     """
#     #     Adds a new directory where data should be find
#     #     :param directory: str
#     #     :param do_update: bool
#     #     """
#     #
#     #     if directory not in self._directories:
#     #         self._directories.append(directory)
#     #         if do_update:
#     #             self.update_data_classes()
#
#     def set_directories(self, directories):
#         """
#         Sets the directories where data should be find
#         :param directories: list(str)
#         """
#
#         new_dir = False
#         for d in directories:
#             if d not in self._directories:
#                 new_dir = True
#                 self._directories.append(d)
#
#         if new_dir:
#             self.update_data_classes(do_reload=True)
#             self.update_data_items(do_reload=True)
#
#     # def update_data_classes(self, do_reload=False):
#     #     """
#     #
#     #     :param do_reload:
#     #     :return:
#     #     """
#     #
#     #     if not self._loaded_data_classes or do_reload:
#     #         for d in self._directories:
#     #             self._loaded_data_classes.extend(self._load_data_classes(d))
#     #
#     #     return self._loaded_data_classes
#
#     def update_data_items(self, do_reload=False):
#         """
#         Adds custom dat files located in the current data manager registered directories
#         """
#
#         if not self._loaded_data_items or do_reload:
#             for d in self._directories:
#                 self._loaded_data_items.extend(self._load_data_items(d))
#             for data_item in self._loaded_data_items:
#                 self.register_item(data_item)
#
#         return self._loaded_data_items
#
#     def get_type_instance(self, data_type):
#         """
#         Returns a new instance of data type
#         :param data_type: str, type of data type instance we want to create
#         :return: variant
#         """
#
#         for data_class in self.get_all_data_classes():
#             if data_class.is_type_match(data_type):
#                 return data_class()
#
#     def _load_data_classes(self, directory):
#
#         imported = list()
#
#         if directory is None or not os.path.exists(directory):
#             LOGGER.warning('Data Path {} does not exists!'.format(directory))
#             return imported
#
#         for loader, mod_name, is_package in pkgutil.walk_packages([directory]):
#             try:
#                 module = loader.find_module(mod_name).load_module(mod_name)
#                 for cname, obj in inspect.getmembers(module, inspect.isclass):
#                     if not issubclass(obj, core_data.FileData):
#                         continue
#                     # globals()[cname] = obj
#                     imported.append(obj)
#             except Exception as exc:
#                 LOGGER.warning('Aborting loading Data Class {} : {}'.format(mod_name, str(exc)))
#                 LOGGER.debug(traceback.format_exc())
#
#         # TODO: Not working in Python 3
#         # return sorted(list(set(imported)))
#
#         return list(set(imported))
#
#     def _load_data_items(self, directory):
#         """
#         Internal function that loads data classes
#         :param directory: str
#         :return: list
#         """
#
#         data_classes = list()
#
#         if directory is None or not path.is_dir(directory):
#             LOGGER.warning('Data Path {} does not exists!'.format(directory))
#             return data_classes
#
#         for loader, mod_name, is_package in pkgutil.walk_packages([directory]):
#             try:
#                 module = loader.find_module(mod_name).load_module(mod_name)
#                 for cname, obj in inspect.getmembers(module, inspect.isclass):
#                     if not issubclass(obj, data.DataItem):
#                         continue
#                     data_classes.append(obj)
#             except Exception as exc:
#                 LOGGER.warning('Aborting loading Data Class {} : {}'.format(mod_name, str(exc)))
#                 LOGGER.error(traceback.format_exc())
#
#         for data_cls in data_classes:
#             LOGGER.info('Found Data Class: {}'.format(data_cls))
#
#         # TODO: Not working in Python 3
#         # '<' not supported between instances of 'Shiboken.ObjectType' and 'Shiboken.ObjectType'
#         # return sorted(list(set(data_classes)))
#         return list(set(data_classes))



#! /usr/bin/env python
# -*- coding: utf-8 -*-

"""
Module that contains factory class to create tpRigToolkit options
"""

from tpDcc.libs.python import python
from tpDcc.libs.options.core import factory

from tpRigToolkit.widgets.options import bone, control, bonecontrollink


def add_option(option_type, name=None, value=None, parent=None, main_widget=None, option_object=None):
    if option_type == 'rigcontrol':
        new_option = _add_control_rig(
            name=name, value=value, parent=parent, main_widget=main_widget, option_object=option_object)
    elif option_type == 'bone':
        new_option = _add_bone(name=name, value=value, parent=parent, main_widget=main_widget)
    elif option_type == 'boneList':
        new_option = _add_bone_list(name=name, value=value, parent=parent, main_widget=main_widget)
    elif option_type == 'boneControlLink':
        new_option = _add_bone_control_link(
            name=name, value=value, parent=parent, main_widget=main_widget, option_object=option_object)
    else:
        new_option = factory.add_option(option_type, name, value, parent, main_widget, option_object)

    return new_option


def _add_control_rig(name='rigcontrol', value=None, parent=None, main_widget=None, option_object=None):
    if type(name) == bool:
        name = 'rigcontrol'

    control_option = control.RigControlOption(
        name=name, parent=parent, main_widget=main_widget, rig_object=option_object)
    control_option.set_value(value)

    return control_option


def _add_bone(name='bone', value=None, parent=None, main_widget=None):
    if type(name) == bool:
        name = 'bone'

    bone_option = bone.BoneOption(name=name, parent=parent, main_widget=main_widget)
    bone_option.set_value(value)

    return bone_option


def _add_bone_list(name='boneList', value=None, parent=None, main_widget=None):
    if type(name) == bool:
        name = 'boneList'

    value = python.force_list(value)
    bone_list_option = bone.BoneOptionList(name=name, parent=parent, main_widget=main_widget)
    bone_list_option.set_value(value)

    return bone_list_option


def _add_bone_control_link(name='boneControlLink', value=None, parent=None, main_widget=None, option_object=None):
    if type(name) == bool:
        name = 'boneControlLink'

    bone_control_link_option = bonecontrollink.BoneControlLinkOption(
        name=name, parent=parent, main_widget=main_widget, rig_object=option_object)
    bone_control_link_option.set_value(value or list())

    return bone_control_link_option



#! /usr/bin/env python
# -*- coding: utf-8 -*-

"""
Module that contains widget to show custom rig options
"""

from __future__ import print_function, division, absolute_import

from tpDcc.libs.options.core import viewer

from tpRigToolkit.widgets.options import rigoptionlist


class RigOptionsViewer(viewer.OptionsViewer, object):

    OPTION_LIST_CLASS = rigoptionlist.RigOptionList

    def __init__(self, option_object=None, settings=None, parent=None):
        super(RigOptionsViewer, self).__init__(option_object=option_object, settings=settings, parent=parent)



#! /usr/bin/env python
# -*- coding: utf-8 -*-

"""
Module that contains widget to handle editable options for rigs
"""

from __future__ import print_function, division, absolute_import

from tpDcc.libs.options.core import option


class RigOption(option.Option, object):
    def __init__(self, name, parent, main_widget, rig_object):
        self._rig = rig_object
        super(RigOption, self).__init__(name=name, parent=parent, main_widget=main_widget)



#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""
Module that contains core classes for tools
"""

from __future__ import print_function, division, absolute_import

import logging.config

from Qt.QtCore import Qt

from tpDcc.libs.qt.core import base

LOGGER = logging.getLogger('tpRigToolkit-core')


class ToolAttacher(object):
    Window = 0
    Dialog = 1


class Tool(base.BaseWidget, object):

    ATTACHER_TYPE = ToolAttacher.Window

    def __init__(self, config, parent=None):

        self._config = config
        self._attacher = None

        super(Tool, self).__init__(parent=parent)

    @property
    def config(self):
        return self._config

    def set_attacher(self, attacher):
        """
        Sets the attacher this tool will be linked to
        :param attacher:
        """

        self._attacher = attacher
        if attacher:
            self.post_attacher_set()

    def close_tool_attacher(self):
        """
        Closes tool attacher and the tool itself
        """

        if not self._attacher:
            self.close_tool()

        if self.ATTACHER_TYPE == ToolAttacher.Window:
            self._attacher.fade_close()
        elif self.ATTACHER_TYPE == ToolAttacher.Dialog:
            self._attacher.fade_close()
        else:
            self.close_tool()

    def post_attacher_set(self):
        """
        Function that is called once an attacher has been set
        Override in child classes
        """

        pass

    def settings(self):
        """
        Returns settings of the attacher
        :return: QtSettings
        """

        if not self._attacher:
            return None

        return self.attacher.settings()

    def save_settings(self):
        """
        Save settings of the attacher
        """

        if not self._attacher:
            return None

        return self._attacher.save_settings()

    def menu_bar(self):
        """
        Returns attacher menu bar
        :return: QMenuBar
        """

        if not self._attacher:
            return None

        return self._attacher.menuBar()

    def add_dock(self, name, **kwargs):
        """
        Adds a new dockable widget to the window
        :param name: str, name of the dock widget
        :return: QDockWidget
        """

        if not self._attacher:
            return None

        if not hasattr(self._attacher, 'add_dock'):
            return None

        return self._attacher.add_dock(name=name, **kwargs)

    def add_toolbar(self, name, area=None):
        """
        Adds a new toolbar to the window
        :return:  QToolBar
        """

        if not self._attacher:
            return None

        if not hasattr(self._attacher, 'add_toolbar'):
            return None

        area = area or Qt.TopToolBarArea

        return self._attacher.add_toolbar(name=name, area=area)

    def register_callback(self, callback_type, fn):
        """
        Registers the given callback with the given function
        :param callback_type: tpDcc.DccCallbacks
        :param fn: Python function to be called when callback is emitted
        """

        if not self._attacher or not hasattr(self._attacher, 'register_callback'):
            return None

        return self._attacher.register_callback(callback_type, fn)

    def unregister_callbacks(self):
        """
        Unregisters all callbacks registered by this tool
        """

        if not self._attacher or not hasattr(self._attacher, 'unregister_callbacks'):
            return None

        return self._attacher.unregister_callbacks()

    def show_ok_message(self, msg):
        """
        Shows an ok message in the attacher
        :param msg: str
        """

        if not self._attacher:
            return

        if not hasattr(self._attacher, 'show_ok_message'):
            LOGGER.warning(
                'Tool Attacher for "{}" has no show_ok_message available!'.format(self.__class__.__name__))

        LOGGER.info(msg)
        self._attacher.show_ok_message(msg)

    def show_warning_message(self, msg):
        """
        Shows a warning message in the attacher
        :param msg: str
        """

        if not self._attacher:
            return

        if not hasattr(self._attacher, 'show_warning_message'):
            LOGGER.warning(
                'Tool Attacher for "{}" has no show_warning_message available!'.format(self.__class__.__name__))

        LOGGER.warning(msg)
        self._attacher.show_warning_message(msg)

    def show_error_message(self, msg):
        """
        Shows an error message in the attacher
        :param msg: str
        """

        if not self._attacher:
            return

        if not hasattr(self._attacher, 'show_error_message'):
            LOGGER.warning(
                'Tool Attacher f>or "{}" has no show_error_message available!'.format(self.__class__.__name__))

        LOGGER.error(msg)
        self._attacher.show_error_message(msg)

    def close_tool(self):
        """
        Close tool
        """

        self.unregister_callbacks()
        self.close()



#! /usr/bin/env python
# -*- coding: utf-8 -*-

"""
Module that contains Space Switch data
"""

from __future__ import print_function, division, absolute_import

from tpDcc.core import data
from tpDcc.libs.qt.widgets.library import savewidget

from tpRigToolkit.core import data as rig_data


class SpaceSwitchFileData(data.CustomData, object):
    def __init__(self, name=None, path=None):
        super(SpaceSwitchFileData, self).__init__(name=name, path=path)

    @staticmethod
    def get_data_type():
        return 'dcc.spaceswitch'

    @staticmethod
    def get_data_extension():
        return 'sswitch'

    @staticmethod
    def get_data_title():
        return 'Space Switch'


class SpaceSwitchPreviewWidget(rig_data.DataPreviewWidget, object):
    def __init__(self, item, parent=None):
        super(SpaceSwitchPreviewWidget, self).__init__(item=item, parent=parent)


class SpaceSwitchSaveWidget(savewidget.BaseSaveWidget, object):
    def __init__(self, item, settings, temp_path=None, parent=None):
        super(SpaceSwitchSaveWidget, self).__init__(item, settings, temp_path, parent=parent)

    def ui(self):
        super(SpaceSwitchSaveWidget, self).ui()

        from tpRigToolkit.tools.spaceswitcher.widgets import spaceswitcher

        self._space_switcher = spaceswitcher.SpaceSwitcherWidget()
        self._extra_layout.addWidget(self._space_switcher)


class SpaceSwitch(rig_data.DataItem, object):
    Extension = '.{}'.format(SpaceSwitchFileData.get_data_extension())
    Extensions = ['.{}'.format(SpaceSwitchFileData.get_data_extension())]
    MenuOrder = 6
    MenuName = SpaceSwitchFileData.get_data_title()
    MenuIconName = 'spaceswitch.png'
    TypeIconName = 'spaceswitch.png'

    TypeIconName = 'spaceswitch.png'
    DataType = SpaceSwitchFileData.get_data_type()
    DefaultDataFileName = 'new_space_switch'
    CreateWidgetClass = SpaceSwitchSaveWidget
    PreviewWidgetClass = SpaceSwitchPreviewWidget

    def __init__(self, *args, **kwargs):
        super(SpaceSwitch, self).__init__(*args, **kwargs)

        self.set_data_class(SpaceSwitchFileData)



#! /usr/bin/env python
# -*- coding: utf-8 -*-

"""
Module that contains widget to handle editable options for rigs
"""

from __future__ import print_function, division, absolute_import

from functools import partial

from Qt.QtCore import Qt, Signal
from Qt.QtWidgets import QSizePolicy, QAction

from tpDcc.managers import resources
from tpDcc.libs.python import name as name_utils
from tpDcc.libs.qt.widgets import layouts, messagebox

from tpDcc.libs.options.core import optionlist

from tpRigToolkit.widgets.options import factory


class RigOptionList(optionlist.OptionList, object):

    FACTORY_CLASS = factory

    def __init__(self, parent=None, option_object=None):
        self._menu_added = False

        super(RigOptionList, self).__init__(parent=parent, option_object=option_object)

        self._option_group_class = RigOptionListGroup

    def _create_context_menu(self, menu, parent=None):
        create_menu = super(RigOptionList, self)._create_context_menu(menu=menu, parent=parent)

        if self._menu_added:
            return create_menu

        control_icon = resources.icon('rigcontrol')
        bone_icon = resources.icon('bone')
        link_icon = resources.icon('link')

        create_menu.addSeparator()
        add_rig_control_action = QAction(control_icon, 'Add Rig Control', create_menu)
        create_menu.addAction(add_rig_control_action)
        add_bone_action = QAction(bone_icon, 'Add Rig Joint', create_menu)
        create_menu.addAction(add_bone_action)
        add_bone_list_action = QAction(bone_icon, 'Add Rig Joint List', create_menu)
        create_menu.addAction(add_bone_list_action)
        add_control_bone_link_action = QAction(link_icon, 'Add Control/Joint Link', create_menu)
        create_menu.addAction(add_control_bone_link_action)

        add_rig_control_action.triggered.connect(partial(parent._add_option, 'rigcontrol'))
        add_bone_action.triggered.connect(partial(parent._add_option, 'bone'))
        add_bone_list_action.triggered.connect(partial(parent._add_option, 'boneList'))
        add_control_bone_link_action.triggered.connect(partial(parent._add_option, 'boneControlLink'))

        self._menu_added = True

        return create_menu


class RigOptionListGroup(RigOptionList, object):
    updateValues = Signal(object)
    widgetClicked = Signal(object)

    def __init__(self, name, option_object, parent=None):
        self._name = name
        super(RigOptionListGroup, self).__init__(option_object=option_object, parent=parent)

        self.setObjectName(name)
        self._original_background_color = self.palette().color(self.backgroundRole())
        self._option_type = self.get_option_type()
        self.supress_select = False
        self.setSizePolicy(QSizePolicy.Minimum, QSizePolicy.Minimum)

    def mousePressEvent(self, event):
        super(RigOptionListGroup, self).mousePressEvent(event)

        if not event.button() == Qt.LeftButton:
            return

        half = self.width() * 0.5
        if event.y() > 25 and event.x() > (half - 50) and event.x() < (half + 50):
            return

        parent = self.get_parent()
        if parent:
            parent.supress_select = True
        if self.supress_select:
            self.supress_select = False
            return

        self.widgetClicked.emit(self)

    def setup_ui(self):
        main_group_layout = layouts.VerticalLayout()
        main_group_layout.setContentsMargins(0, 0, 0, 0)
        main_group_layout.setSpacing(1)
        self.group = optionlist.OptionGroup(self._name)
        self.child_layout = self.group.child_layout

        self.main_layout = layouts.VerticalLayout()
        self.main_layout.setContentsMargins(0, 0, 0, 0)
        self.main_layout.setSpacing(0)
        self.main_layout.addSpacing(2)
        self.main_layout.addWidget(self.group)
        self.setLayout(self.main_layout)

        self.group.expand.connect(self._on_expand_updated)
    #
    # def _create_context_menu(self, menu, parent):
    #     pass

    def get_name(self):
        """
        Returns option group name
        :return: str
        """

        return self.group.title()

    def set_name(self, name):
        """
        Sets option group name
        :param name: str
        """

        self.group.setTitle(name)

    def get_option_type(self):
        """
        Returns the type of the option
        :return: str
        """

        return 'group'

    def get_value(self):
        """
        Returns whether group is expanded or not
        :return: bool
        """

        expanded = not self.group.is_collapsed()
        return expanded

    def get_children(self):
        """
        Returns all group Options
        :return: list(Option)
        """

        item_count = self.child_layout.count()
        found = list()
        for i in range(item_count):
            item = self.child_layout.itemAt(i)
            widget = item.widget()
            found.append(widget)

        return found

    def set_expanded(self, flag):
        """
        Sets the expanded/collapsed state of the group
        :param flag: bool
        """

        if flag:
            self.expand_group()
        else:
            self.collapse_group()

    def expand_group(self):
        """
        Expands group
        """

        self.group.expand_group()

    def collapse_group(self):
        """
        Collapse gorup
        """

        self.group.collapse_group()

    def save(self):
        """
        Function that saves the current state of the group option
        :return:
        """
        self._write_options(clear=False)

    def rename(self, new_name=None):
        """
        Function that renames group
        :param new_name: variant, str or None
        """

        found = self._get_widget_names()
        title = self.group.title()
        if not new_name:
            new_name, button = messagebox.MessageBox.input(
                self._parent, 'Rename Option', 'Type a new option name:', input_text=title)
            if not new_name or new_name == title:
                return

        while new_name in found:
            new_name = name_utils.increment_last_number(new_name)

        self.group.setTitle(new_name)
        self._write_all()

    def move_up(self):
        """
        Function that moves up selected Options
        """

        parent = self.parent()
        layout = parent.child_layout
        index = layout.indexOf(self)
        if index == 0:
            return
        index -= 1
        parent.child_layout.removeWidget(self)
        layout.insertWidget(index, self)

        self._write_all()

    def move_down(self):
        """
        Function that moves down selected options
        """

        parent = self.parent()
        layout = parent.child_layout
        index = layout.indexOf(self)
        if index == (layout.count() - 1):
            return
        index += 1
        parent.child_layout.removeWidget(self)
        layout.insertWidget(index, self)

        self._write_all()

    def copy_to(self, parent):
        """
        Function that copy selected options into given parent
        :param parent: Option
        """

        group = parent.add_group(self.get_name(), parent)
        children = self.get_children()
        for child in children:
            if child == group:
                continue
            child.copy_to(group)

    def remove(self):
        """
        Function that removes selected options
        :return:
        """
        parent = self.parent()
        if self in self._parent._current_widgets:
            remove_index = self._parent._current_widgets.index(self)
            self._parent._current_widgets.pop(remove_index)
        parent.child_layout.removeWidget(self)
        self.deleteLater()
        self._write_all()

    def _on_expand_updated(self, value):
        self.updateValues.emit(False)



#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""
Base wrapper classes to create DCC windows
"""

from __future__ import print_function, division, absolute_import

import webbrowser

from Qt.QtCore import Qt, QSize
from Qt.QtWidgets import QApplication, QSizePolicy, QWidget, QPushButton, QMainWindow, QToolBar

from tpDcc import dcc
from tpDcc.dcc import window
from tpDcc.managers import resources
from tpDcc.libs.qt.core import qtutils, statusbar
from tpDcc.libs.qt.widgets import layouts


class WindowStatusBar(statusbar.StatusWidget, object):
    def __init__(self, parent=None):
        super(WindowStatusBar, self).__init__(parent)

        self._info_url = None
        self._tool = None

        self.setFixedHeight(25)
        self._info_btn = QPushButton()
        self._info_btn.setIconSize(QSize(25, 25))
        self._info_btn.setSizePolicy(QSizePolicy(QSizePolicy.Fixed, QSizePolicy.Fixed))
        self._info_btn.setIcon(resources.icon('info1'))
        self._info_btn.setStyleSheet('QWidget {background-color: rgba(255, 255, 255, 0); border:0px;}')

        self._bug_btn = QPushButton()
        self._bug_btn.setIconSize(QSize(25, 25))
        self._bug_btn.setSizePolicy(QSizePolicy.Fixed, QSizePolicy.Fixed)
        self._bug_btn.setIcon(resources.icon('bug'))
        self._bug_btn.setStyleSheet('QWidget {background-color: rgba(255, 255, 255, 0); border:0px;}')

        self.main_layout.insertWidget(0, self._info_btn)
        self.main_layout.insertWidget(1, self._bug_btn)

        self._info_btn.clicked.connect(self._on_open_url)
        self._bug_btn.clicked.connect(self._on_send_bug)

    def set_info_url(self, url):
        """
        Sets the URL used to open tool info documentation web
        :param url: str
        """

        self._info_url = url

    def set_tool(self, tool):
        """

        :param tool:
        :return:
        """

        self._tool = tool

    def has_url(self):
        """
        Returns whether the URL documentation web is set or not
        :return: bool
        """

        if self._info_url:
            return True

        return False

    def has_tool(self):
        """
        Returns whether window has a tool attached or not
        :return: bool
        """

        if self._tool:
            return True

        return False

    def show_info(self):
        """
        Shows the info button of the status bar
        """

        self._info_btn.setVisible(True)

    def hide_info(self):
        """
        Hides the info button of the status bar
        """

        self._info_btn.setVisible(False)

    def show_bug(self):
        self._bug_btn.setVisible(True)

    def hide_bug(self):
        self._bug_btn.setVisible(False)

    def open_info_url(self):
        """
        Opens tool documentation URL in user web browser
        """

        if not self._project:
            return False

        if self._info_url:
            webbrowser.open_new_tab(self._info_url)

    def _on_send_bug(self):

        if not self._project:
            return False

        # tools.ToolsManager().run_tool(self._project, 'bugtracker', extra_args={'tool': self._tool})

    def _on_open_url(self):
        """
        Internal callback function that is called when the user presses the info icon button
        :return:
        """

        self.open_info_url()


class MainWindow(window.Window, object):

    LOGO_NAME = None
    STATUS_BAR_WIDGET = WindowStatusBar

    def __init__(
            self,
            tool=None,
            name='MainWindow',
            title='MainWindow',
            size=(800, 535),
            fixed_size=False,
            parent=None,
            *args,
            **kwargs):

        self._tool = tool

        super(MainWindow, self).__init__(
            name=name,
            title=title,
            size=size,
            fixed_size=fixed_size,
            auto_run=True,
            frame_less=True,
            use_style=False,
            parent=parent,
            *args,
            **kwargs
        )

        if self.parent():
            for widget in self.parent().findChildren(QMainWindow):
                if widget is not self:
                    if widget.objectName() == self.objectName():
                        widget.close()

        screen_geo = QApplication.desktop().screenGeometry()
        screen_width = screen_geo.width()
        screen_height = screen_geo.height()
        new_width = self.width()
        new_height = self.height()
        if self.width() > screen_width:
            new_width = 500
        if self.height() > screen_height:
            new_height = 500
        self.resize(new_width, new_height)
        self.center()

    def ui(self):
        super(MainWindow, self).ui()

        window_icon = self._get_icon()
        self.setWindowIcon(window_icon)

        title_layout = layouts.HorizontalLayout(spacing=0, margins=(0, 0, 0, 0))
        title_layout.setAlignment(Qt.AlignTop | Qt.AlignHCenter)
        self.main_layout.insertLayout(0, title_layout)

        self._status_bar.set_tool(self._tool)
        if not self._status_bar.has_url():
            self._status_bar.hide_info()
        if not self._status_bar.has_tool():
            self._status_bar.hide_bug()

        if self._tool:
            self.main_layout.addWidget(self._tool)
        #
        # if self._project.is_dev():
        #     int_colors = self._project.dev_color0.split(',')
        #     dev_style = "background-color: rgb({}, {}, {})".format(
        #         int_colors[0], int_colors[1], int_colors[2], 255)
        #     self._dragger.setStyleSheet(dev_style)

    def setWindowTitle(self, title):
        # if self._project.is_dev():
        #     title = '{} - [{}]'.format(title, self._project.get_environment())

        super(MainWindow, self).setWindowTitle(title)

    def closeEvent(self, event):
        if self._tool:
            self._tool.close_plugin()
        super(MainWindow, self).closeEvent(event)

    def add_toolbar(self, name, area=Qt.TopToolBarArea):
        """
        Overrides base MainWindow add_toolbar function
        :param name: str
        :param area:
        :return:
        """

        new_toolbar = QToolBar(name)
        # The 0 widget is always the header view of the window
        self.main_layout.insertWidget(1, new_toolbar)
        return new_toolbar

    def set_info_url(self, url):
        """
        Sets the info URL of the current window
        :param url: str
        """

        if not url:
            return

        self._status_bar.set_info_url(url)

        if not self._status_bar.has_url():
            self._status_bar.hide_info()
        else:
            self._status_bar.show_info()

    def _get_icon(self):
        return resources.icon('tprigtoolkit')


def dock_window(window_class, min_width=300):
    """
    Utility function to dock Maya window
    :param window_class: cls
    """

    if not dcc.is_maya():
        return

    import maya.cmds as cmds
    import maya.OpenMayaUI as OpenMayaUI
    try:
        cmds.deleteUI(window_class.name)
    except Exception:
        pass

    main_control = cmds.workspaceControl(
        window_class.name, ttc=["AttributeEditor", -1], iw=min_width, mw=True, wp='preferred', label=window_class.title)

    control_widget = OpenMayaUI.MQtUtil.findControl(window_class.name)
    control_wrap = qtutils.wrapinstance(int(control_widget), QWidget)
    control_wrap.setAttribute(Qt.WA_DeleteOnClose)
    win = window_class(parent=control_wrap)

    cmds.evalDeferred(lambda *args: cmds.workspaceControl(main_control, e=True, rs=True))

    win.show()

    return win



# #! /usr/bin/env python
# # -*- coding: utf-8 -*-
#
# """
# Module that contains implementation for node manager
# """
#
# from __future__ import print_function, division, absolute_import
#
# import os
# import pkgutil
# import logging
# import traceback
# from collections import defaultdict
#
# from tpDcc.libs.python import python, decorators
#
# from tpRigToolkit import packages
# # from tpRigToolkit.core import package
#
# LOGGER = logging.getLogger('tpRigToolkit')
#
# """
# TODO
# This is class is related to PyFlow packages implementation. We do not need although before removing it we should
# check if this can be useful for the RigNode implementation
# """
#
#
# ENV_VAR_NAME = 'TPRIGTOOLKIT_PACKAGE_PATHS'
# ENV_VAR_PATHS_DELIMITER = ';'
#
# # def __init__(self, package_paths=None):
# #     super(PackagesManager, self).__init__()
#
# _REGISTERED_MODULES = dict()
# _REGISTERED_MODULE_PATHS = dict()
# _REGISTERED_UI_NODES_FACTORIES = dict()
# _REGISTERED_UI_PORTS_FACTORIES = dict()
# _REGISTERED_INPUT_WIDGETS_FACTORIES = dict()
# _REGISTERED_TOOLS = defaultdict(list)
# _REGISTERED_PREFS = defaultdict(list)
#
# # self._module_paths = python.force_list(package_paths) if package_paths else list()
# # self._update_module_paths_from_environment()
#
#
# def registered_modules():
#     """
#     Returns dict with the registered modules
#     :return: dict
#     """
#
#     return self._registered_modules
#
#
# def registered_modules_paths():
#     """
#     Returns dict containing registered modules paths
#     :return: dict
#     """
#
#     return self._registered_module_paths
#
#
# def registered_ui_nodes_factories():
#     """
#     Returns dict containing registered UI nodes factories
#     :return: dict
#     """
#
#     return self._registered_ui_nodes_factories
#
#
# def registered_ui_ports_factories():
#     """
#     Returns dict containing registered UI ports factories
#     :return: dict
#     """
#
#     return self._registered_ui_ports_factories
#
#
# def registered_input_widgets_factories():
#     """
#     Returns dict containing registered input widgets
#     :return: dict
#     """
#
#     return self._registered_input_widgets_factories
#
#
# def registered_tools():
#     """
#     Returns dict containing registered tools
#     :return: dict
#     """
#
#     return self._registered_tools
#
#
# def registered_prefs():
#     """
#     Returns dict containing registered preference widgets
#     :return: dict
#     """
#
#     return self._registered_prefs
#
#
# def register_module_path(module_path):
#     """
#     Registers given path into the manager so path is taking into account when loading tpNodeGrpah modules
#     :param module_path: str
#     """
#
#     if not module_path or not os.path.isdir(module_path) or module_path in self._module_paths:
#         return
#
#     self._module_paths.append(module_path)
#     self._load_modules(module_path)
#
#
# def register_ui_node_factory(module_name, factory):
#     """
#     Registers given factory and maps it to given module name
#     :param module_name: str
#     :param factory: Factory
#     """
#
#     if module_name not in self._registered_ui_nodes_factories:
#         self._registered_ui_nodes_factories[module_name] = factory
#
#
# def register_ui_port_factory(module_name, factory):
#     """
#     Registers given factory and maps it to given module name
#     :param module_name: str
#     :param factory: Factory
#     """
#
#     if module_name not in self._registered_ui_ports_factories:
#         self._registered_ui_ports_factories[module_name] = factory
#
#
# def register_input_widgets_factory(module_name, factory):
#     """
#     Registers given factory and maps it to given module name
#     :param module_name: str
#     :param factory: Factory
#     """
#
#     if module_name not in self._registered_input_widgets_factories:
#         self._registered_input_widgets_factories[module_name] = factory
#
#
# def register_tool(module_name, tool_class):
#     """
#     Registers given tool class and maps it to given module name
#     :param module_name: str
#     :param tool_class: cls
#     """
#
#     registered_tool_names = [tool.NAME for tool in self._registered_tools[module_name]]
#     if tool_class.NAME not in registered_tool_names:
#         self._registered_tools[module_name].append(tool_class)
#         tool_class.MODULE_NAME = module_name
#
#
# def register_preference_widget(module_name, preference_widget_class):
#     """
#     Registers given preference widget class and maps it to the given module name
#     :param module_name: str
#     :param preference_widget_class: cls
#     """
#
#     registered_prefs_categories = [pref.CATEGORY for pref in self._registered_prefs[module_name]]
#     if preference_widget_class.CATEGORY not in registered_prefs_categories:
#         self._registered_prefs[module_name].append(preference_widget_class)
#         preference_widget_class.MODULE_NAME = module_name
#
#
# def initialize():
#     import PyFlow
#     _load_modules()
#
#
# def _load_modules(module_path=None):
#     """
#     Function that loads all the modules located in the registered paths
#     """
#
#     if not module_path:
#         module_paths = self._module_paths
#     else:
#         module_paths = [module_path]
#
#     for importer, mod_name, is_pkg in pkgutil.iter_modules(module_paths):
#         try:
#             if is_pkg:
#                 mod = importer.find_module(mod_name).load_module(mod_name)
#                 mod_path = mod.__path__[0]
#                 if hasattr(mod, 'MODULE_NAME'):
#                     module_name = mod.MODULE_NAME
#                 else:
#                     module_name = mod_name
#                 mod_class = type(module_name, (package.Package,), {})
#                 new_module = mod_class(mod_path)
#                 self._registered_modules[module_name] = new_module
#                 self._registered_module_paths[module_name] = os.path.normpath(mod_path)
#         except Exception as e:
#             LOGGER.error('Error on Module: {} : \n{}'.format(mod_name, str(e)))
#             LOGGER.error(traceback.format_exc())
#             continue
#
#     registered_internal_port_data_types = set()
#
#     for name, mod in self._registered_modules.items():
#         module_name = mod.__class__.__name__
#
#         for n in mod.node_classes.values():
#             n.MODULE_NAME = module_name
#
#         for p in mod.port_classes.values():
#             p.MODULE_NAME = module_name
#             if p.is_value_port():
#                 internal_type = p.internal_data_structure()
#                 if internal_type in registered_internal_port_data_types:
#                     raise Exception(
#                         'Port with "{}" internal data type already been registered!'.format(internal_type))
#                 registered_internal_port_data_types.add(internal_type)
#
#         ui_nodes_factory = mod.ui_nodes_factory_classes
#         if ui_nodes_factory:
#             self.register_ui_node_factory(module_name, ui_nodes_factory)
#
#         ui_ports_factory = mod.ui_ports_factory_classes
#         if ui_ports_factory:
#             self.register_ui_port_factory(name, ui_ports_factory)
#
#         input_widgets_factory = mod.input_widgets_factory_classes
#         if input_widgets_factory:
#             self.register_input_widgets_factory(name, input_widgets_factory)
#
#         for tool_class in mod.tool_classes.values():
#             supported_softwares = tool_class.SUPPORTED_SOFTWARES
#             if 'any' not in supported_softwares:
#                 continue
#             self.register_tool(name, tool_class)
#
#         for pref_widget_class in mod.preference_widget_classes.values():
#             self.register_preference_widget(name, pref_widget_class)
#
#
# def _update_module_paths_from_environment():
#     """
#     Internal function that updates registered module paths by taking into account current environment variables
#     """
#
#     def _recurse_module_paths(module_path):
#         """
#         Recursively loops through given path searching modules
#         :param module_path: str
#         :return: list(str)
#         """
#
#         paths_found = list()
#         for sub_folder in os.listdir(module_path):
#             sub_folder_path = os.path.join(module_path, sub_folder)
#             if os.path.isdir(sub_folder_path):
#                 if sub_folder_path.endswith('Package') or sub_folder_path.endswith('package'):
#                     paths_found.append(sub_folder_path)
#
#         return paths_found
#
#     module_paths = packages.__path__
#
#     if self.ENV_VAR_NAME in os.environ:
#         extra_paths = os.environ.get(self.ENV_VAR_NAME).rstrip(self.ENV_VAR_PATHS_DELIMITER)
#         for modules_root in extra_paths.split(self.ENV_VAR_PATHS_DELIMITER):
#             if os.path.isdir(modules_root):
#                 paths = _recurse_module_paths(modules_root)
#                 extra_paths.extend(paths)
#
#         module_paths.extend(extra_paths)
#
#     for p in module_paths:
#         self.register_module_path(p)



#! /usr/bin/env python
# -*- coding: utf-8 -*-

"""
Module that contains script editor tool implementation
"""

from __future__ import print_function, division, absolute_import

from Qt.QtCore import Qt, Signal
from Qt.QtWidgets import QSizePolicy, QWidget

from tpDcc.core import plugin
from tpDcc.managers import resources
from tpDcc.libs.qt.widgets import layouts
from tpDcc.tools.scripteditor.core import client, tool, toolset


class ScriptsEditorPlugin(plugin.DockPlugin, object):

    NAME = 'Scripts Editor'
    TOOLTIP = 'Allow to edit scripts easily'
    DEFAULT_DOCK_AREA = Qt.RightDockWidgetArea
    IS_SINGLETON = True

    def __init__(self):
        super(ScriptsEditorPlugin, self).__init__()

        self._script_editor_widget = None
        self._content = QWidget()
        self._content_layout = layouts.VerticalLayout(spacing=0, margins=(0, 0, 0, 0))
        self._content.setLayout(self._content_layout)
        self.setWidget(self._content)

    @staticmethod
    def icon():
        return resources.icon('source_code')

    def show_plugin(self):
        super(ScriptsEditorPlugin, self).show_plugin()

        if not self._script_editor_widget:

            # Settings are used to colorize script editor text depending on current theme
            settings = self._app.settings()
            self._script_editor_widget = ScriptEditorWidget(settings=settings, parent=self)
            # self._script_editor_widget = scripteditor.ScriptEditorWidget(settings=settings, load_session=False)
            # self._script_editor_widget.scriptSaved.connect(self._on_script_saved)
            self._script_editor_widget.setSizePolicy(QSizePolicy.Expanding, QSizePolicy.Expanding)
            self._content_layout.addWidget(self._script_editor_widget)
            self._script_editor_widget.editorClosed.connect(self.close)

    def load_script(self, script_file):
        if not self._script_editor_widget:
            return

        self._script_editor_widget.controller.load_script(script_file)

    def _on_script_saved(self, file_path):
        pass


class ScriptEditorWidget(toolset.ScriptEditorToolset):

    editorClosed = Signal()

    def __init__(self, *args, **kwargs):
        kwargs['load_session'] = False
        super(ScriptEditorWidget, self).__init__(*args, **kwargs)

        # NOTE: Important to store client in a variable to avoid Python garbage collector to delete it
        self._script_editor_client = client.ScriptEditorClient.create_and_connect_to_server(tool.ScriptEditorTool.ID)

        self.initialize(client=self._script_editor_client)

        self._title_frame.setVisible(False)

        self.model.console_visible = False
        self.model.menubar_visible = False
        self.model.toolbar_visible = False
        # self.model.enable_save_script = False

        self.controller.close_all_scripts()

        self.view.lastTabClosed.connect(self.editorClosed.emit)



#! /usr/bin/env python
# -*- coding: utf-8 -*-

"""
Module that contains options to link bones/joints with controls
"""

from __future__ import print_function, division, absolute_import

from Qt.QtCore import Qt, Signal

from tpDcc.libs.qt.core import base
from tpDcc.libs.qt.widgets import layouts, buttons
from tpDcc.libs.options.options import list

from tpRigToolkit.widgets.options import bone, control


class BoneControlLinkOption(list.ListOption, object):
    def __init__(self, name, parent, main_widget, rig_object):
        self._rig = rig_object
        super(BoneControlLinkOption, self).__init__(name=name, parent=parent, main_widget=main_widget)

    def get_option_type(self):
        return 'boneControlLink'

    def get_option_widget(self):
        return GetBoneControlLinkWidget(name=self._name, rig_object=self._rig)

    def get_value(self):
        list_value = self._option_widget.get_value()

        return list_value


class GetBoneControlLinkWidget(list.GetListWidget, object):
    def __init__(self, name, rig_object, parent=None):
        self._rig = rig_object
        super(GetBoneControlLinkWidget, self).__init__(name=name, parent=parent)

    def get_list_widget(self):
        return BoneControlLinkList(rig_object=self._rig)


class BoneControlLinkList(list.ListWidget, object):
    def __init__(self, rig_object):
        self._rig = rig_object
        super(BoneControlLinkList, self).__init__()

    def _get_entry_widget(self, name):
        return BoneControlLinkItem(self._rig, name)

    def _build_entry(self, link_info=None):
        entry_widget = self._get_entry_widget(link_info)
        entry_widget.itemRemoved.connect(self._cleanup_garbage)
        entry_widget.valueChanged.connect(self._on_value_changed)
        entry_widget.itemDuplicated.connect(self._on_duplicated_item)

        return entry_widget

    def _on_duplicated_item(self, widget):
        value = widget.get_value()
        if not value:
            return
        self.add_entry(value)

        self._on_value_changed()


class BoneControlLinkItem(base.BaseWidget, object):
    valueChanged = Signal(object, object)
    itemRemoved = Signal(object)
    itemDuplicated = Signal(object)

    def __init__(self, rig_object, link_info=None, parent=None):
        self._rig = rig_object
        link_info = link_info if link_info else dict()
        self._control_data = link_info.get('control', dict())
        self._bone_name = link_info.get('node', '')
        self._garbage = None
        super(BoneControlLinkItem, self).__init__(parent=parent)

    def get_main_layout(self):
        main_layout = layouts.HorizontalLayout()
        main_layout.setAlignment(Qt.AlignRight)
        main_layout.setSpacing(0)
        main_layout.setContentsMargins(0, 0, 0, 0)

        return main_layout

    def ui(self):
        super(BoneControlLinkItem, self).ui()

        self._control_line = control.ControlLineEdit(rig_object=self._rig)
        self._bone_line = bone.BoneLineEdit()
        self._duplicate_btn = buttons.BaseToolButton().image('clone').icon_only()
        self._remove_btn = buttons.BaseToolButton().image('delete').icon_only()

        self._control_line.set_data(self._control_data)
        self._bone_line.setText(self._bone_name)

        self.main_layout.addWidget(self._control_line)
        self.main_layout.addWidget(self._bone_line)
        self.main_layout.addSpacing(10)
        self.main_layout.addWidget(self._duplicate_btn)
        self.main_layout.addWidget(self._remove_btn)

    def setup_signals(self):
        self._control_line.controlSelected.connect(self._on_selected_control)
        self._duplicate_btn.clicked.connect(self._on_duplicate_item)
        self._remove_btn.clicked.connect(self._on_remove_item)
        self._control_line.controlSelected.connect(self._on_value_changed)
        self._bone_line.textChanged.connect(self._on_value_changed)

    def get_value(self):
        control_data = self._control_line.control_data
        selected_bone = self._bone_line.selected_node

        return {
            'control': control_data,
            'node': selected_bone
        }

    def _on_selected_control(self, control_data):
        self._control_data = control_data or dict()
        self._control_line.set_data(self._control_data)

    def _on_remove_item(self):
        self._garbage = True
        self.itemRemoved.emit(self)

    def _on_duplicate_item(self):
        self._garbage = True
        self.itemDuplicated.emit(self)

    def _on_value_changed(self):
        control_data = self._control_line.control_data
        bone_node = self._bone_line.selected_node
        self.valueChanged.emit(control_data, bone_node)



#! /usr/bin/env python
# -*- coding: utf-8 -*-

"""
Initialization module for tpRigToolkit
"""

from __future__ import print_function, division, absolute_import

import os
import logging.config

import tpDcc.loader as dcc_loader
from tpDcc.libs.python import contexts
from tpDcc.core import dcc as core_dcc
from tpDcc.managers import configs, resources, libs, tools, menus
from tpDcc.libs.qt.managers import toolsets as qt_toolsets

import tpRigToolkit.config
import tpRigToolkit.toolsets

# =================================================================================

PACKAGE = 'tpRigToolkit'

# =================================================================================


def init(import_libs=True, dev=False):
    """
    Initializes tpRigToolkit module
    :param import_libs: bool, Whether to import deps libraries by default or not
    :param dev: bool, Whether tpRigToolkit is initialized in dev mode or not
    """

    logger = create_logger(dev=dev)

    if import_libs:
        dcc_loader.init(dev=dev)

    dcc_loader_module = core_dcc.get_dcc_loader_module('tpRigToolkit.dccs')
    if dcc_loader_module:
        dcc_loader_module.init(dev=dev)

    register_resources()

    # Register configuration files
    configs.register_package_configs(PACKAGE, os.path.dirname(tpRigToolkit.config.__file__))
    core_config = configs.get_config('tpRigToolkit-core', environment='development' if dev else 'production')
    if not core_config:
        logger.warning(
            'tpRigToolkit-core configuration file not found! '
            'Make sure that you have tpRigToolkit-config package installed!')
        return None

    libs_to_load = core_config.get('libs', list())
    tools_to_load = core_config.get('tools', list())

    with contexts.Timer('Libraries loaded', logger=logger):
        libs.LibsManager().register_package_libs(PACKAGE, libs_to_register=libs_to_load)

    with contexts.Timer('Tools loaded', logger=logger):
        tools.ToolsManager().register_package_tools(PACKAGE, tools_to_register=tools_to_load)

    with contexts.Timer('Toolsets loaded', logger=logger):
        qt_toolsets.ToolsetsManager().register_toolset_file_path(
            os.path.dirname(os.path.abspath(tpRigToolkit.toolsets.__file__)), package_name=PACKAGE)
        qt_toolsets.ToolsetsManager().load_registered_toolsets(package_name=PACKAGE, tools_to_load=tools_to_load)

    with contexts.Timer('Menu created', logger=logger):
        menus.create_menus(package_name=PACKAGE, dev=dev)


def create_logger(dev=False):
    """
    Returns logger of current module
    """

    logger_dir = os.path.normpath(os.path.join(os.path.expanduser('~'), 'tpRigToolkit', 'logs'))
    if not os.path.isdir(logger_dir):
        os.makedirs(logger_dir)

    logging_config = os.path.normpath(os.path.join(os.path.dirname(__file__), '__logging__.ini'))

    logging.config.fileConfig(logging_config, disable_existing_loggers=False)
    logger = logging.getLogger('tpRigToolkit')
    if dev:
        logger.setLevel(logging.DEBUG)
        for handler in logger.handlers:
            handler.setLevel(logging.DEBUG)

    return logger


def register_resources():
    """
    Registers tpDcc.libs.qt resources path
    """

    resources_path = os.path.join(os.path.dirname(os.path.abspath(__file__)), 'resources')
    resources.register_resource(resources_path, key='tpRigToolkit-core')


create_logger()



#! /usr/bin/env python
# -*- coding: utf-8 -*-

"""
Module that contains project widgets implementation for tpRigToolkit
"""

from __future__ import print_function, division, absolute_import

import os
import logging

from Qt.QtCore import Qt, Signal, QSize
from Qt.QtWidgets import QSizePolicy, QWidget, QLabel, QListWidget, QListWidgetItem, QMenu
from Qt.QtGui import QPixmap

from tpDcc.managers import resources, tools
from tpDcc.libs.python import python, path as path_utils
from tpDcc.libs.qt.core import base, qtutils
from tpDcc.libs.qt.widgets import layouts, dividers, buttons, combobox, directory, tabs

from tpRigToolkit.widgets.options import rigoptionsviewer

LOGGER = logging.getLogger('tpRigToolkit-core')


class ProjectSettingsWidget(base.BaseWidget, object):
    exitSettings = Signal()

    def __init__(self, project=None, parent=None):
        self._project = None
        super(ProjectSettingsWidget, self).__init__(parent=parent)

        self._fill_version_types_combo()

        if project:
            self.set_project(project)

    def ui(self):
        super(ProjectSettingsWidget, self).ui()

        image_layout = layouts.HorizontalLayout(spacing=2, margins=(2, 2, 2, 2))
        image_layout.setContentsMargins(2, 2, 2, 2)
        image_layout.setSpacing(2)
        self.main_layout.addLayout(image_layout)
        self._project_image = QLabel()
        self._project_image.setAlignment(Qt.AlignCenter)
        image_layout.addStretch()
        image_layout.addWidget(self._project_image)
        image_layout.addStretch()

        self._settings_tab = tabs.BaseTabWidget(parent=self)
        self.main_layout.addWidget(self._settings_tab)

        self._naming_widget = NamingWidget(project=self._project)
        self._project_options_widget = rigoptionsviewer.RigOptionsViewer(option_object=self._project, parent=self)
        self._external_code_widget = ExternalCodeDirectoryWidget(parent=self)
        version_control_widget = QWidget(parent=self)
        version_control_layout = layouts.VerticalLayout(spacing=0, margins=(2, 2, 2, 2))
        version_control_widget.setLayout(version_control_layout)
        self._version_type_combo = combobox.BaseComboBox(parent=self)
        version_control_layout.addWidget(self._version_type_combo)
        version_control_layout.addStretch()

        self._settings_tab.addTab(self._project_options_widget, 'Settings')
        self._settings_tab.addTab(self._naming_widget, 'Nomenclature')
        self._settings_tab.addTab(version_control_widget, 'Version Control')
        self._settings_tab.addTab(self._external_code_widget, 'External Code')

        bottom_layout = layouts.VerticalLayout(spacing=2, margins=(2, 2, 2, 2))
        bottom_layout.setAlignment(Qt.AlignBottom)
        self.main_layout.addLayout(bottom_layout)
        bottom_layout.addLayout(dividers.DividerLayout())

        buttons_layout = layouts.HorizontalLayout(spacing=2, margins=(2, 2, 2, 2))
        bottom_layout.addLayout(buttons_layout)

        ok_icon = resources.icon('ok')
        self._ok_btn = buttons.BaseButton(parent=self)
        self._ok_btn.setIcon(ok_icon)
        buttons_layout.addWidget(self._ok_btn)

    def setup_signals(self):
        self._version_type_combo.currentIndexChanged.connect(self._on_version_control_type_changed)
        self._ok_btn.clicked.connect(self._on_exit)

    def get_project(self):
        """
        Returns current RigBuilder project used by this widget
        :return: Project
        """

        return self._project

    def set_project(self, project):
        """
        Sets current project used by this widget
        :param project: Project
        """

        self._project = project

        self._project_options_widget.set_option_object(self._project)
        if self._project:
            self._project_image.setPixmap(
                QPixmap(self._project.get_project_image()).scaled(QSize(150, 150), Qt.KeepAspectRatio))
        self._naming_widget.set_project(self._project)
        self._external_code_widget.set_project(self._project)
        if self._project and self._project.settings.has_setting('version_control'):
            try:
                version_control_index = int(self._project.settings.has_setting('version_control'))
            except Exception:
                version_control_index = 0
            self._version_type_combo.setCurrentIndex(version_control_index)

    def update_options(self, do_reload=False):
        """
        Update options of the current project
        """

        if not self._project:
            return

        if do_reload:
            self._project.reload_options()

        self._project_options_widget.update_options()

    def _fill_version_types_combo(self):
        """
        Internal callback function that fills with the different types of supported version controls
        """

        self._version_type_combo.clear()
        for version_type in ['none', 'git']:
            self._version_type_combo.addItem(resources.icon(version_type), version_type)

    def _on_version_control_type_changed(self, index):
        """
        Internal callback function that is called when version control type is updated
        :param index: int
        :return:
        """

        if not self._project:
            return

        self._project.settings.set('version_control', int(index))

    def _on_exit(self):
        """
        Internal callback function that is called when the user exists settings widget
        """

        self.exitSettings.emit()


class NamingWidget(base.BaseWidget, object):
    def __init__(self, project=None, parent=None):

        self._project = project

        super(NamingWidget, self).__init__(parent=parent)

        self.update_rules()

    def get_main_layout(self):
        main_layout = layouts.VerticalLayout(spacing=2, margins=(0, 0, 0, 0))

        return main_layout

    def ui(self):
        super(NamingWidget, self).ui()

        naming_layout = layouts.HorizontalLayout(spacing=2, margins=(0, 0, 0, 0))
        edit_icon = resources.icon('edit')
        name_lbl = QLabel('Naming Rule: ')
        self._name_rules = combobox.BaseComboBox(parent=self)
        self._name_rules.setSizePolicy(QSizePolicy.Expanding, QSizePolicy.Fixed)
        self._edit_btn = buttons.BaseButton(parent=self)
        self._edit_btn.setIcon(edit_icon)
        naming_layout.addWidget(name_lbl)
        naming_layout.addWidget(self._name_rules)
        naming_layout.addWidget(self._edit_btn)

        self.main_layout.addLayout(naming_layout)
        self.main_layout.addStretch()

    def setup_signals(self):
        self._name_rules.currentIndexChanged.connect(self._on_update_rule)
        self._edit_btn.clicked.connect(self._on_open_naming_manager)

    def set_project(self, project):
        self._project = project
        self.update_rules()

    def update_rules(self):

        try:
            self._name_rules.blockSignals(True)

            self._name_rules.clear()
            if not self._project:
                return
            naming_lib = self._project.naming_lib
            if not naming_lib:
                return
            naming_lib.load_session()
            rules = naming_lib.rules
            for rule in rules:
                self._name_rules.addItem(rule.name, userData=rule)
            rule_name = self._set_rule()
            self._name_rules.setCurrentText(rule_name)
        except Exception as exc:
            LOGGER.warning('Error while updating rules: {}'.format(exc))
        finally:
            self._name_rules.blockSignals(False)

    def _set_rule(self, rule=None):
        if not self._project:
            return
        naming_lib = self._project.naming_lib
        if not naming_lib:
            return

        if rule:
            rule_name = rule.name
            if self._project.settings.has_setting('naming_rule'):
                current_rule = self._project.settings.get('naming_rule')
                if current_rule == rule_name:
                    return
                self._project.settings.set('naming_rule', rule_name)
        else:
            if not self._project.settings.has_setting('naming_rule'):
                self._project.settings.set('naming_rule', self._name_rules.currentText())

        if not self._project.settings.has_setting('naming_rule'):
            return
        rule_name = self._project.settings.get('naming_rule')
        if not naming_lib.has_rule(rule_name):
            return
        naming_lib.set_active_rule(rule_name)

        return rule_name

    def _on_update_rule(self, index):
        rule = self._name_rules.itemData(index)
        self._set_rule(rule=rule)

    def _on_open_naming_manager(self):
        # naming_manager_tool = tools.ToolsManager().launch_tool_by_id(
        #     'tpRigToolkit-tools-namemanager', project=self._project)
        naming_manager_tool = tools.ToolsManager().launch_tool_by_id(
            'tpDcc-tools-nameit', naming_lib=self._project.naming_lib)
        attacher = naming_manager_tool.attacher
        attacher.closed.connect(self.update_rules)


class ExternalCodeList(QListWidget):

    directoriesChanged = Signal(object)

    def __init__(self, parent=None):
        super(ExternalCodeList, self).__init__(parent)

        self.setAlternatingRowColors(True)
        self.setSelectionMode(self.NoSelection)
        self.setContextMenuPolicy(Qt.CustomContextMenu)
        self.customContextMenuRequested.connect(self._on_item_menu)

        self._create_context_menu()

    def get_directories(self):
        count = self.count()
        found = list()
        if not count:
            return found
        for i in range(count):
            item = self.item(i)
            if not item:
                continue
            found.append(str(item.text()))

        return found

    def refresh(self, directories):
        directories = python.force_list(directories)
        self.clear()
        for diretory_found in directories:
            name = diretory_found
            if not path_utils.is_dir(diretory_found):
                name = 'Directory Not Valid! {}'.format(diretory_found)
            item = QListWidgetItem()
            item.setText(name)
            item.setSizeHint(QSize(20, 25))
            self.addItem(item)

    def _create_context_menu(self):
        self._context_menu = QMenu()
        remove_action = self._context_menu.addAction('Remove')
        remove_action.setIcon(resources.icon('trash'))
        remove_action.triggered.connect(self._on_remove_action)

    def _on_item_menu(self, pos):
        item = self.itemAt(pos)
        if not item:
            return
        self._context_menu.exec_(self.viewport().mapToGlobal(pos))

    def _on_remove_action(self):
        index = self.currentIndex()
        self.takeItem(index.row())
        self.directoriesChanged.emit(self.get_directories())


class ExternalCodeDirectoryWidget(directory.GetDirectoryWidget):
    def __init__(self, project=None, parent=None):
        self._project = None
        super(ExternalCodeDirectoryWidget, self).__init__(parent)

        self.set_project(project)

    def get_main_layout(self):
        return layouts.VerticalLayout(spacing=2, margins=(2, 2, 2, 2))

    def ui(self):
        super(ExternalCodeDirectoryWidget, self).ui()

        self._directory_widget.setVisible(False)
        self._directory_widget.setEnabled(False)

        self._list = ExternalCodeList(parent=self)

        file_layout = layouts.HorizontalLayout()
        self._directory_browse_button = buttons.BaseButton('Add External Code Path', parent=self)
        self._directory_browse_button.setIcon(resources.icon('folder'))
        file_layout.addWidget(self._directory_browse_button)

        self.main_layout.addWidget(self._list)
        self.main_layout.addLayout(file_layout)

    def setup_signals(self):
        super(ExternalCodeDirectoryWidget, self).setup_signals()

        self._list.directoriesChanged.connect(self._on_send_directories)
        self._directory_browse_button.clicked.connect(self._on_browse)

    def set_directory(self, directory):
        directory = python.force_list(directory)
        self._last_directory = self._directory
        self._directory = directory
        self._list.refresh(directory)

    def set_project(self, project):
        self._project = project
        if not self._project:
            self._list.clear()
            return
        external_code_paths = self._project.get_external_paths(relative=False) or list()
        self._list.refresh(external_code_paths)

    def get_external_code_paths(self):
        return self._list.get_directories()

    def set_external_code_paths(self, paths_list):
        self._list.refresh(paths_list)

    def _on_browse(self):
        projects_path = self._project.full_path if self._project else None
        if not projects_path or not os.path.isdir(projects_path):
            projects_path = 'C:/'

        filename = qtutils.get_folder(projects_path, parent=self)
        if not filename:
            return
        filename = path_utils.clean_path(filename)
        if filename and path_utils.is_dir(filename):
            self._on_text_changed(filename)

    def _on_text_changed(self, directory):
        directory = str(directory)
        if not path_utils.is_dir(directory):
            return
        found = self._list.get_directories()
        if directory in found:
            return
        if found:
            found.insert(0, directory)
        else:
            found = [directory]
        self.directoryChanged.emit(found)
        self._list.refresh(found)

        if self._project:
            self._project.set_external_code_paths(found)

    def _on_send_directories(self, directories):
        self.directoryChanged.emit(directories)
        if self._project:
            self._project.set_external_code_paths(directories)



#! /usr/bin/env python
# -*- coding: utf-8 -*-

"""
Module that contains constants definitions for tpRigToolkit
"""

from __future__ import print_function, division, absolute_import

DEFAULT_SIDES = {'center': 'c', 'left': 'l', 'right': 'r'}
DEFAULT_SIDE = 'center'
DEFAULT_MIRROR_SIDE = 'right'



#! /usr/bin/env python
# -*- coding: utf-8 -*-

"""
Module that contains options to select rig bones/joints
"""

from __future__ import print_function, division, absolute_import

import os
import json

from Qt.QtCore import Qt, QFileInfo
from Qt.QtWidgets import QDialog, QTreeWidget, QTreeWidgetItem

from tpDcc import dcc
from tpDcc.libs.qt.core import base
from tpDcc.libs.qt.widgets import layouts, lineedit, buttons, dividers
from tpDcc.libs.options.core import option
from tpDcc.libs.options.options import list, text


class BoneOption(option.Option, object):
    def __init__(self, name, parent, main_widget):
        super(BoneOption, self).__init__(name=name, parent=parent, main_widget=main_widget)

    def get_option_type(self):
        return 'bone'

    def get_option_widget(self):
        return GetBoneWidget(name=self._name)

    def get_value(self):
        value = self._option_widget.get_text()
        if not value:
            value = ''

        return value

    def set_value(self, value):
        value = str(value)
        self._option_widget.set_text(value)

    def _setup_option_widget_value_change(self):
        self._option_widget.textChanged.connect(self._on_value_changed)


class BoneOptionList(list.ListOption, object):
    def __init__(self, name, parent, main_widget):
        super(BoneOptionList, self).__init__(name=name, parent=parent, main_widget=main_widget)

    def get_option_type(self):
        return 'boneList'


class GetBoneWidget(text.TextWidget, object):
    def __init__(self, name='', parent=None):
        super(GetBoneWidget, self).__init__(name=name, parent=parent)

    def get_text_widget(self):
        return BoneLineEdit()


class BoneLineEdit(lineedit.BaseLineEdit, object):
    def __init__(self, text='', parent=None):
        super(BoneLineEdit, self).__init__(text=text, parent=parent)

    @property
    def selected_node(self):
        return self.text()

    def dropEvent(self, event):
        if event.mimeData().hasUrls():
            data = event.mimeData().urls()
            file_info = QFileInfo(data[0].toLocalFile())
            file_name = file_info.absoluteFilePath()
            if file_name and os.path.isfile(file_name):
                file_extension = os.path.splitext(file_name)[-1]
                data_extension = skeleton.SkeletonFileData.get_data_extension()
                if not data_extension.startswith('.'):
                    data_extension = '.{}'.format(data_extension)
                if file_extension == data_extension:
                    self._show_bones_hierarchy(file_name)
                    event.accept()
        elif event.mimeData().hasText():
            self.setText(event.mimeData().text())

        event.accept()

    def _show_bones_hierarchy(self, file_path):
        dlg = QDialog(parent=dcc.get_main_window() or None)
        dlg.setWindowTitle('Select Skeleton Node')
        lyt = layouts.VerticalLayout(spacing=0, margins=(0, 0, 0, 0))
        dlg.setLayout(lyt)
        bone_hierarchy_widget = BoneHierarchyWidget(file_path, parent=dlg)
        current_bone = self.text() or ''
        bone_hierarchy_widget.set_bone(current_bone)
        lyt.addWidget(bone_hierarchy_widget)
        dlg.exec_()
        selected_node = bone_hierarchy_widget.selected_node
        if not selected_node:
            return
        self.setText(selected_node)


class BoneHierarchyWidget(base.BaseWidget, object):
    def __init__(self, file_path, parent=None):
        self._file_path = file_path
        self._selected_node = None
        super(BoneHierarchyWidget, self).__init__(parent=parent)

        self._load_data()

    @property
    def selected_node(self):
        return self._selected_node

    def ui(self):
        super(BoneHierarchyWidget, self).ui()

        self._tree_hierarchy = QTreeWidget()
        self._node_line = lineedit.BaseLineEdit(parent=self)
        self._node_line.setReadOnly(True)
        self._ok_btn = buttons.BaseButton('Ok')
        self._cancel_btn = buttons.BaseButton('Cancel')
        buttons_layout = layouts.HorizontalLayout()
        buttons_layout.addStretch()
        buttons_layout.addWidget(self._ok_btn)
        buttons_layout.addWidget(self._cancel_btn)

        self.main_layout.addWidget(self._tree_hierarchy)
        self.main_layout.addWidget(self._node_line)
        self.main_layout.addWidget(dividers.Divider())
        self.main_layout.addLayout(buttons_layout)

    def setup_signals(self):
        self._tree_hierarchy.currentItemChanged.connect(self._on_item_selected)
        self._ok_btn.clicked.connect(self._on_ok)
        self._cancel_btn.clicked.connect(self._on_cancel)

    def set_bone(self, bone_name):
        if not bone_name:
            return

        find_items = self._tree_hierarchy.findItems(bone_name, Qt.MatchExactly | Qt.MatchRecursive, 0)
        if not find_items:
            return

        find_item = find_items[0]
        find_item.setSelected(True)
        self._tree_hierarchy.setCurrentItem(find_item)
        self._tree_hierarchy.scrollTo(self._tree_hierarchy.indexFromItem(find_item))

    def _load_data(self):
        self._tree_hierarchy.clear()
        if not self._file_path or not os.path.isfile(self._file_path):
            return

        with open(self._file_path, 'r') as fh:
            skeleton_data = json.load(fh)
        if not skeleton_data:
            return

        created_nodes = dict()
        for node_data in skeleton_data:
            node_index = node_data.get('index', 0)
            node_parent_index = node_data.get('parent_index', -1)
            node_name = node_data.get('name', 'new_node')
            new_node = QTreeWidgetItem()
            new_node.setText(0, node_name)
            created_nodes[node_index] = {'node': new_node, 'parent_index': node_parent_index}
        sorted(created_nodes.items(), key=lambda x: int(x[0]))

        for node_index, node_data in created_nodes.items():
            node_data = created_nodes.get(node_index, None)
            if not node_data:
                continue
            node_item = node_data.get('node')
            parent_index = node_data['parent_index']
            if parent_index <= -1:
                self._tree_hierarchy.addTopLevelItem(node_item)
                continue
            parent_node_data = created_nodes.get(parent_index, None)
            if not parent_node_data:
                continue
            parent_node_item = parent_node_data.get('node')
            parent_node_item.addChild(node_item)

        self._tree_hierarchy.expandAll()

    def _on_item_selected(self, current, previous):
        node_name = current.text(0)
        self._node_line.setText(node_name)
        self._selected_node = node_name

    def _on_ok(self):
        self.parent().close()

    def _on_cancel(self):
        self._selected_node = None
        self.parent().close()
